package com.adc.da.bs.customized.dao;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import lombok.extern.slf4j.Slf4j;

import org.hibernate.transform.Transformers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.adc.da.bs.customized.enums.CustomizedServiceEnum;
import com.adc.da.util.exception.AdcDaBaseException;
import com.adc.da.util.utils.StringUtils;

/**
 * 定制服务dao
 *
 * @author 韩磊
 * @date 2019/3/21 17:25
 */
@Slf4j
@Component
public class CustomizedDao {
    protected final EntityManager entityManager;

    private static final String USER_ID = "userId";

    @Autowired
    public CustomizedDao(EntityManager entityManager) {
        super();
        this.entityManager = entityManager;
    }

    /**
     * 插入定制服务数据
     *
     * @param map
     * @return int
     * @author 韩磊
     * @date 2019/3/22 9:42
     */
    public int createCustomizedService(Map<String, Object> map) {
        String inSql =
                " INSERT INTO NERDS.TS_ADCBS_CUSTOMIZED_SERVICE (SERVICE_ID,PHONE_NUM, CONTACTER, SERVICE_TYPE ," +
                        "CONTENT, HANDLE_STATUS, APPLICANT_ID, APPLICANT_DATE, ENABLE,CREATE_DATE,CREATE_ID," +
                        "UPDATE_DATE,UPDATE_ID ) " +
                        "VALUES (NERDS.retrieval_seq.nextval, ?, ?, ?, ?, ?, ? , SYSDATE,?, SYSDATE, ?, SYSDATE ,?) ";
        Query inQuery = entityManager.createNativeQuery(inSql);
        inQuery.setParameter(1, map.get("phoneNum"));
        inQuery.setParameter(2, map.get("contacter"));
        inQuery.setParameter(3, map.get("serviceType"));
        inQuery.setParameter(4, map.get("content"));
        inQuery.setParameter(5, 0);
        inQuery.setParameter(6, map.get(USER_ID));
        inQuery.setParameter(7, 0);
        inQuery.setParameter(8, map.get(USER_ID));
        inQuery.setParameter(9, map.get(USER_ID));

        int i = 0;
        try {
            i = inQuery.executeUpdate();
        } catch (Exception e) {
            log.error(e.toString());
            throw new AdcDaBaseException("存储定制服务请求失败：" + map.get("pname"), e.toString());
        }
        return i;
    }

    /**
     * 获取对应登录人的定制服务记录
     * 如果是adc登录，则可以看到所有数据
     *
     * @param usermap
     * @return java.util.List<java.util.Map   <   java.lang.String   ,   java.lang.Object>>
     * @author 韩磊
     * @date 2019/3/22 9:42
     */
    public List<Map<String, Object>> getCustomizedServiceHistoryList(Map<String, Object> usermap) {
        StringBuilder sql = new StringBuilder();
        sql.append(" SELECT sevice.SERVICE_ID as serviceId, sevice.PHONE_NUM as phoneNum, ");
        sql.append(" sevice.CONTACTER as contacter, sevice.CONTENT as content,");
        sql.append(" users.name as name, sevice.APPLICANT_DATE as applicantDate,  ");
        sql.append(" sevice.SERVICE_TYPE as serviceType  ");
        sql.append(" FROM NERDS.TS_ADCBS_CUSTOMIZED_SERVICE sevice ");
        sql.append(" LEFT JOIN RETRIEVAL.RET_USERS users ON users.ID = sevice.APPLICANT_ID");
        sql.append(" WHERE 1=1  ");
        if (!"61077".equals(usermap.get("ID").toString()) || !"adc".equals(usermap.get("USERNAME").toString())) {
            sql.append("  AND sevice.APPLICANT_ID = ?  ");
        }
        sql.append("   ORDER BY sevice.APPLICANT_DATE DESC  ");
        Query query = entityManager.createNativeQuery(sql.toString());
        query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
        if (!"61077".equals(usermap.get("ID").toString()) || !"adc".equals(usermap.get("USERNAME").toString())) {
            query.setParameter(1, usermap.get("ID"));
        }
        List<Map<String, Object>> list = query.getResultList();
        for (Map<String, Object> serviceMap : list) {
            List<String> typeList = Arrays.asList(serviceMap.get("SERVICETYPE").toString().split(","));
            List<String> typeNameList = new CopyOnWriteArrayList<>();
            for(String type : typeList){
                typeNameList.add(CustomizedServiceEnum.getServiceEnumNameByValue(Integer.parseInt(type)));
            }
            serviceMap.put("SERVICETYPE", StringUtils.join(typeNameList,","));
        }
        return list;
    }
}
