package com.adc.da.bs.industrymap.rest;

import com.adc.da.bs.industrymap.service.IndustryMapService;
import com.adc.da.bs.industrymap.vo.BarVO;
import com.adc.da.bs.industrymap.vo.CoDistributePieVO;
import com.adc.da.bs.industrymap.vo.MapVO;
import com.adc.da.bs.industrymap.vo.VehicleDetailVO;
import com.adc.da.util.http.PageInfo;
import com.adc.da.util.http.PageParam;
import com.adc.da.util.http.ResponseMessage;
import com.adc.da.util.http.Result;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * 产业地图
 *
 * @author 韩磊
 */
@RestController
@RequestMapping(value = "/${restPath}/map")
@Api(description = "产业地图")
public class IndustryMapController {

    @Autowired
    private IndustryMapService industryMapService;

    /**
     * 地图悬浮展示
     * 整车企业分布
     *
     * @return
     * @ahtor 韩磊
     */
    @ApiOperation(value = "最新批次申报情况")
    @GetMapping("/map")
    public ResponseMessage<List<MapVO>> queryMapAmount() {
        return Result.success(industryMapService.queryMapAmount());
    }

    /**
     * 最新批次申报情况
     * 默认为公告最新批次新产品申报情况，最新批次选“批次列最大值”
     *
     * @param areaName
     * @return
     * @ahtor 韩磊
     */
    @ApiOperation(value = "最新批次新产品申报情况和新能源推荐目录新产品申报情况")
    @GetMapping("/latest/batch/declaration")
    public ResponseMessage<List<BarVO>> queryLatestBatchDeclaration(String areaName,Integer newEnergy,String ym) {
        return Result.success(industryMapService.queryLatestBatchDeclaration(areaName,newEnergy,ym));
    }

    /**
     * 企业分布
     *
     * @param areaName
     * @return java.util.Map<java.lang.String                               ,                               java.lang.Object>
     * @author 韩磊
     * @date 2019/7/30 17:19
     */
    @ApiOperation(value = "企业分布")
    @GetMapping("/vehicle/distribution")
    public ResponseMessage<CoDistributePieVO> queryVehicleDistribution(String areaName,Integer newEnergy,String ym) {
        return Result.success(industryMapService.queryVehicleDistribution(areaName,newEnergy,ym));
    }

    /**
     * 企业分布 -详情
     *
     * @param areaName
     * @return java.util.Map<java.lang.String                               ,                               java.lang.Object>
     * @author 韩磊
     * @date 2019/7/30 17:19
     */
    @ApiOperation(value = "整车企业分布-详情")
    @GetMapping("/vehicle/detail")
    public ResponseMessage<PageInfo<VehicleDetailVO>> queryVehicleDetail(String areaName,Integer newEnergy, PageParam page) {
    	if(null!=page && page.getOffset() > 0){
    		page.setOffset(page.getOffset()-10);
    	}
    	
    	return Result.success(industryMapService.queryVehicleDetail(areaName,newEnergy,page));
    }

    
    /**
     * 企业排行
     *
     */
    @ApiOperation(value = "企业排行")
    @GetMapping("/vehicle/enterpriseRanking")
    public ResponseMessage<List<BarVO>> enterpriseRanking(String type,String type1,String startTime, String endTime) {
        return Result.success(industryMapService.enterpriseRanking(type,type1,startTime,endTime));
    }
    /**
     * 区域排行
     *
     */
    @ApiOperation(value = "区域排行")
    @GetMapping("/vehicle/provinceRanking")
    public ResponseMessage<List<BarVO>> provinceRanking(String type,String type1,String startTime, String endTime) {
        return Result.success(industryMapService.provinceRanking(type,type1,startTime,endTime));
    }
    /**
     * 区域排行
     *
     */
    @ApiOperation(value = "品牌排行")
    @GetMapping("/vehicle/brandRanking")
    public ResponseMessage<List<BarVO>> brandRanking(String type,String type1,String startTime, String endTime) {
        return Result.success(industryMapService.brandRanking(type,type1,startTime,endTime));
    }
    /**
     * 区域排行
     *
     */
    @ApiOperation(value = "最新批次")
    @GetMapping("/vehicle/queryBatch")
    public ResponseMessage<Map<String, Object>> queryBatch() {
        return Result.success(industryMapService.queryBatch());
    }
}
