package com.adc.da.bs.manage.dao;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.apache.commons.lang3.StringUtils;
import org.hibernate.transform.Transformers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.adc.da.bs.manage.entity.RecordManageEntity;
import com.adc.da.util.http.PageParam;

/**
 * 4.5.	备案经验管理
 * @author LuZhiPeng
 *
 */
@Component
public class RecordManageDao {
	protected final EntityManager entityManager;
    @Autowired
    public RecordManageDao(EntityManager entityManager) {
        super();
        this.entityManager = entityManager;
    }
    /**
     * 分页并按上传日期（降序）显示所有记录
     * @param page
     * @return
     */
    public List<Map<String,Object>> findAllByPage(PageParam page){
    	StringBuffer sql = new StringBuffer();
    	sql.append("SELECT R.ID, R.EXP_NAME, R.EXP_DES, R.TYPE_NUM, to_char(R.UPLOAD_DATE,'yyyy-MM-dd HH24:mi:ss'), R.UPLOAD_FILES,");
    	sql.append("R.DEL_NUM, to_char(R.DELETE_DATE,'yyyy-MM-dd HH24:mi:ss'), R.CREATOR, R.VIEW_COUNT, R.DOWNLOAD_COUNT ");
    	sql.append("FROM RETRIEVAL.RECORD_MANAGE R WHERE R.DEL_NUM='0' ");    	
    	sql.append("ORDER BY to_char(R.UPLOAD_DATE,'yyyy-MM-dd HH24:mi:ss') DESC");
    	Query query = entityManager.createNativeQuery(sql.toString());
    	query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
    	query.setFirstResult(page.getOffset());
        query.setMaxResults(page.getLimit());
        List<Map<String, Object>> list = query.getResultList();
        return list;
    }
    /**
     * 按条件查询出所有的行数
     * @param type  查询类型
     * @param expName  查询记录标题名称
     * @return
     */
    
    public Long findTotalCount(PageParam page,String type, String expName) {
    	StringBuffer sql = new StringBuffer();
    	sql.append("SELECT COUNT(*) FROM RETRIEVAL.RECORD_MANAGE R WHERE R.DEL_NUM='0' ");
    	if(StringUtils.isNotEmpty(type)){      //判断是否输入类型
    		sql.append("AND R.TYPE_NUM LIKE '%"+type+"%'");   
    	}
    	if(StringUtils.isNotEmpty(expName)) {    //判断是否输入记录标题名称
    		sql.append("AND R.EXP_NAME LIKE '%"+expName+"%'");
    	}
    	if(StringUtils.isNotEmpty(page.getStartTime())) {            //判断是否输入起始时间
    		sql.append("AND to_char(R.UPLOAD_DATE,'yyyy-MM-dd HH24:mi:ss')>='"+page.getStartTime()+"'");
    	}
    	if(StringUtils.isNotEmpty(page.getEndTime())) {            //判断是否输入结束时间
    		sql.append("AND to_char(R.UPLOAD_DATE,'yyyy-MM-dd HH24:mi:ss')<='"+page.getEndTime()+"'");
    	}
    	
    	Query query = entityManager.createNativeQuery(sql.toString());
    	Long count = Long.parseLong(query.getSingleResult().toString());
        return count;
    }
    /**
     * 条件查询记录并分页
     * @param page  页
     * @param type  查询类型
     * @param expName  查询记录标题名称
     * @return
     */   
    public List<Map<String,Object>> findListByPage(PageParam page,String type, String expName){
    	StringBuffer sql = new StringBuffer();
    	sql.append("SELECT R.ID, R.EXP_NAME, R.EXP_DES, R.TYPE_NUM, to_char(R.UPLOAD_DATE,'yyyy-MM-dd HH24:mi:ss'), R.UPLOAD_FILES,");
    	sql.append("R.DEL_NUM, to_char(R.DELETE_DATE,'yyyy-MM-dd HH24:mi:ss'), R.CREATOR, R.VIEW_COUNT, R.DOWNLOAD_COUNT ");
    	sql.append("FROM RETRIEVAL.RECORD_MANAGE R WHERE R.DEL_NUM='0' ");    	
    	if(StringUtils.isNotEmpty(type)){      //判断是否输入类型
    		sql.append("AND R.TYPE_NUM LIKE '%"+type+"%'");   
    	}
    	if(StringUtils.isNotEmpty(expName)) {    //判断是否输入记录标题名称
    		sql.append("AND R.EXP_NAME LIKE '%"+expName+"%'");
    	}
    	if(StringUtils.isNotEmpty(page.getStartTime())) {            //判断是否输入起始时间
    		sql.append("AND to_char(R.UPLOAD_DATE,'yyyy-MM-dd HH24:mi:ss')>='"+page.getStartTime()+"'");
    	}
    	if(StringUtils.isNotEmpty(page.getEndTime())) {            //判断是否输入结束时间
    		sql.append("AND to_char(R.UPLOAD_DATE,'yyyy-MM-dd HH24:mi:ss')<='"+page.getEndTime()+"'");
    	}
    	sql.append("ORDER BY to_char(R.UPLOAD_DATE,'yyyy-MM-dd HH24:mi:ss') DESC");
    	 Query query = entityManager.createNativeQuery(sql.toString());
         query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
         query.setFirstResult(page.getOffset());
         query.setMaxResults(page.getLimit());
         List<Map<String, Object>> list = query.getResultList();
         return list;    	
    }
    
    /**
     * 增加新记录
     * @param workManageEntity
     * @return
     */
    public Map<String,Object> insertWork(RecordManageEntity recordManageEntity){
    	String idSql = "SELECT RETRIEVAL.R_SEQ.nextval from dual";
    	Query idQuery = entityManager.createNativeQuery(idSql);
        Integer id = Integer.parseInt(idQuery.getSingleResult().toString());
    	Map<String,Object> res = new HashMap<>();
    	Date date=new Date();
		SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		String s=sf.format(date);		
    	StringBuffer sql = new StringBuffer();
    	sql.append("INSERT INTO RETRIEVAL.RECORD_MANAGE (ID,EXP_NAME,EXP_DES,TYPE_NUM,UPLOAD_DATE,UPLOAD_FILES,DEL_NUM,CREATOR,VIEW_COUNT,DOWNLOAD_COUNT) VALUES ");
    	sql.append("("+id+",");
    	sql.append("'"+recordManageEntity.getExpName()+"',");
    	sql.append("'"+recordManageEntity.getExpDes()+"',");
    	sql.append("'"+recordManageEntity.getTypeNum()+"',");
    	sql.append(""+"to_date ( '"+s+"', 'YYYY-MM-DD HH24:MI:SS' )"+",");
    	sql.append("'"+recordManageEntity.getUploadFiles()+"',");
    	sql.append(""+recordManageEntity.getDelNum()+",");
    	//sql.append(""+null+",");
    	sql.append("'"+recordManageEntity.getCreator()+"',");
    	sql.append(""+recordManageEntity.getViewCount()+",");
    	sql.append(""+recordManageEntity.getDownloadCount()+")");
    	
    	try {
            Query query = entityManager.createNativeQuery(sql.toString());

            
            if (query.executeUpdate() == 1){
                res.put("success", "1");
                res.put("msg", "保存成功");
            }else{
            	res.put("success", "0");
                res.put("msg", "保存失败");
            }
        } catch (Exception e) {
            res.put("success", "0");
            res.put("msg", e.toString());
            e.printStackTrace();
        }
        return res;
    }
    /**
            * 增加浏览数
     * @param id
     * @return
     */
    public boolean changeViewCountById(Integer id) {
        String updateViewSql = " UPDATE RETRIEVAL.RECORD_MANAGE SET VIEW_COUNT = (VIEW_COUNT+1) WHERE ID = ? ";
        Query updateQuery = entityManager.createNativeQuery(updateViewSql);
        updateQuery.setParameter(1, id);
        if(updateQuery.executeUpdate() == 1){
            return true;
        }
        return false;
    }
    /**
            * 增加下载数
     * @param id
     * @return
     */
    public boolean changeDownloadCountById(Integer id) {
        String updateDownloadSql = " UPDATE RETRIEVAL.RECORD_MANAGE SET DOWNLOAD_COUNT = (DOWNLOAD_COUNT+1) WHERE ID = ? ";
        Query updateQuery = entityManager.createNativeQuery(updateDownloadSql);
        updateQuery.setParameter(1, id);
        if(updateQuery.executeUpdate() == 1){
            return true;
        }
        return false;
    }
    
    /**
     * 逻辑删除记录：RETRIEVAL.RECORD_MANAGE.DEL_NUM=1为删除，RETRIEVAL.RECORD_MANAGE.DEL_NUM=0为可使用
     * @param id
     * @return
     */
    public boolean deleteWork(Integer id) {
    	Date date=new Date();
		SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		String s=sf.format(date);
    	StringBuffer deleteSql = new StringBuffer();
    	deleteSql.append(" UPDATE RETRIEVAL.RECORD_MANAGE SET DEL_NUM = '1',");
    	deleteSql.append("DELETE_DATE="+"to_date ( '"+s+"', 'YYYY-MM-DD HH24:MI:SS' ) ");
    	deleteSql.append(""+"WHERE ID = '"+id+"'");
		Query query = entityManager.createNativeQuery(deleteSql.toString());
        if(query.executeUpdate() == 1 ){
            return true;
        }
        return false;
	}
}
