package com.adc.da.bs.manage.dao;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.apache.commons.lang3.StringUtils;
import org.hibernate.SQLQuery;
import org.hibernate.transform.Transformers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.adc.da.bs.manage.entity.StandardManageEntity;
import com.adc.da.util.http.PageParam;

@Component
public class StandardManageDao {
	protected final EntityManager entityManager;
    @Autowired
    public StandardManageDao(EntityManager entityManager) {
        super();
        this.entityManager = entityManager;
    }
    /**即将生效标准
     * 分页并按上传日期（降序）显示所有记录
     * @param page
     * @return
     */
    public List<Map<String,Object>> findAllByPage(PageParam page){
    	Date date=new Date();
    	SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd");
    	String s=sf.format(date);//创建当前时间（字符串类型） 
    	StringBuffer sql = new StringBuffer(" SELECT ROWNUM RN ,T.* FROM ( ");
    	sql.append("SELECT S.ID, S.STA_NUM, S.SAMPLING, S.NAME, S.TYPE_NUM, S.STATUS, to_char(S.IMPL_DATE,'yyyy-MM-dd') IMPL_DATE, S.URL,");
    	sql.append("S.DEL_NUM, to_char(S.DELETE_DATE,'yyyy-MM-dd HH24:mi:ss'), S.CREATOR, S.VIEW_COUNT, S.DOWNLOAD_COUNT ");
    	sql.append("FROM NERDS.STANDARD_MANAGE S WHERE S.DEL_NUM IS NULL AND S.IMPL_DATE <= SYSDATE");    	
    	sql.append("ORDER BY to_char(S.IMPL_DATE,'yyyy-MM-dd HH24:mi:ss') DESC");
    	sql.append(" ) T ");
    	Query query = entityManager.createNativeQuery(sql.toString());
    	query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
    	query.setFirstResult(page.getOffset());
        query.setMaxResults(page.getLimit());
        List<Map<String, Object>> list = query.getResultList();
        return list;
    }
    
    /**即将生效标准
     * 按条件查询出所有的行数
     * @param type  查询类型
     * @param keyName  查询的关键字
     * @return
     */
    
    public Long findTotalCount(PageParam page,String type, String keyName) {
    	Date date=new Date();
    	SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd");
    	String s=sf.format(date);//创建当前时间（字符串类型）
    	StringBuffer sql = new StringBuffer();
    	sql.append("SELECT COUNT(*) FROM NERDS.STANDARD_MANAGE S WHERE S.DEL_NUM IS NULL  AND S.IMPL_DATE > = SYSDATE ");
    	if(StringUtils.isNotEmpty(type)){      //判断是否输入类别
    		sql.append("AND S.TYPE_NUM LIKE '%"+type+"%'");   
    	}
    	if(StringUtils.isNotEmpty(keyName)) {    //判断是否输入查询的关键字
    		sql.append("AND CONCAT(S.NAME,S.STA_NUM) LIKE '%"+keyName+"%'");
    	}
    	if(StringUtils.isNotEmpty(page.getStartTime())) {            //判断是否输入起始时间
    		sql.append("AND to_char(S.IMPL_DATE,'yyyy-MM-dd HH24:mi:ss')>='"+page.getStartTime()+"'");
    	}
    	if(StringUtils.isNotEmpty(page.getEndTime())) {            //判断是否输入结束时间
    		sql.append("AND to_char(S.IMPL_DATE,'yyyy-MM-dd HH24:mi:ss')<='"+page.getEndTime()+"'");
    	}
    	
    	Query query = entityManager.createNativeQuery(sql.toString());
    	Long count = Long.parseLong(query.getSingleResult().toString());
        return count;
    }
    
    /**即将生效标准
     * 条件查询记录并分页
     * @param page  页
     * @param type  查询类型
     * @param keyName  查询的关键字
     * @return
     */   
    public List<Map<String,Object>> findListByPage(PageParam page,String type, String keyName){
    	Date date=new Date();
    	SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd");
    	String s=sf.format(date);//创建当前时间（字符串类型）
    	StringBuffer sql = new StringBuffer(" SELECT ROWNUM RN ,T.* FROM ( ");
    	sql.append("SELECT S.ID, S.STA_NUM, S.SAMPLING, S.NAME, S.TYPE_NUM, S.STATUS, to_char(S.IMPL_DATE,'yyyy-MM-dd') IMPL_DATE, S.URL,");
    	sql.append("S.DEL_NUM, to_char(S.DELETE_DATE,'yyyy-MM-dd HH24:mi:ss'), S.CREATOR, S.VIEW_COUNT, S.DOWNLOAD_COUNT ");
    	sql.append("FROM NERDS.STANDARD_MANAGE S WHERE S.DEL_NUM IS NULL AND S.IMPL_DATE > SYSDATE ");   	
    	if(StringUtils.isNotEmpty(type)){      //判断是否输入类型
    		sql.append("AND S.TYPE_NUM LIKE '%"+type+"%'");   
    	}
    	if(StringUtils.isNotEmpty(keyName)) {    //判断是否输入查询的关键字
    		sql.append("AND CONCAT(S.NAME,S.STA_NUM) LIKE '%"+keyName+"%'");
    	}
    	if(StringUtils.isNotEmpty(page.getStartTime())) {            //判断是否输入起始时间
    		sql.append("AND to_char(S.IMPL_DATE,'yyyy-MM-dd')>='"+page.getStartTime()+"'");
    	}
    	if(StringUtils.isNotEmpty(page.getEndTime())) {            //判断是否输入结束时间
    		sql.append("AND to_char(S.IMPL_DATE,'yyyy-MM-dd')<='"+page.getEndTime()+"'");
    	}
    	sql.append("ORDER BY to_char(S.IMPL_DATE,'yyyy-MM-dd HH24:mi:ss') ");
    	sql.append(" ) T ");
    	 Query query = entityManager.createNativeQuery(sql.toString());
         query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
         query.setFirstResult(page.getOffset());
         query.setMaxResults(page.getLimit());
         List<Map<String, Object>> list = query.getResultList();
         return list;    	
    }
    
    /**发布历史查询
     * 分页并按上传日期（降序）显示所有记录
     * @param page
     * @return
     */
    public List<Map<String,Object>> findAllByPageHistory(PageParam page){
    	Date date=new Date();
    	SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd");
    	String s=sf.format(date);//创建当前时间（字符串类型）
    	StringBuffer sql = new StringBuffer(" SELECT ROWNUM RN ,T.* FROM ( ");
    	sql.append("SELECT S.ID, S.STA_NUM, S.SAMPLING, S.NAME, S.TYPE_NUM, S.STATUS, to_char(S.IMPL_DATE,'yyyy-MM-dd') IMPL_DATE, S.URL,");
    	sql.append("S.DEL_NUM, to_char(S.DELETE_DATE,'yyyy-MM-dd HH24:mi:ss') IMPL_DATE, S.CREATOR, S.VIEW_COUNT, S.DOWNLOAD_COUNT ");
    	sql.append("FROM NERDS.STANDARD_MANAGE S WHERE S.DEL_NUM IS NULL   S.IMPL_DATE <= SYSDATE ");    	
    	sql.append("ORDER BY to_char(S.IMPL_DATE,'yyyy-MM-dd HH24:mi:ss') DESC");
    	sql.append(" ) T ");
    	Query query = entityManager.createNativeQuery(sql.toString());
    	query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
    	query.setFirstResult(page.getOffset());
        query.setMaxResults(page.getLimit());
        List<Map<String, Object>> list = query.getResultList();
        return list;
    }
    
    /**发布历史查询
     * 按条件查询出所有的行数
     * @param type  查询类型
     * @param keyName  查询的关键字
     * @return
     */
    
    public Long findTotalCountHistory(PageParam page,String type, String keyName) {
    	Date date=new Date();
    	SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd");
    	String s=sf.format(date);//创建当前时间（字符串类型）
    	StringBuffer sql = new StringBuffer();
    	sql.append("SELECT COUNT(*) FROM NERDS.STANDARD_MANAGE S WHERE S.DEL_NUM IS NULL AND  S.IMPL_DATE <= SYSDATE ");
    	if(StringUtils.isNotEmpty(type)){      //判断是否输入类别
    		sql.append("AND S.TYPE_NUM LIKE '%"+type+"%'");   
    	}
    	if(StringUtils.isNotEmpty(keyName)) {    //判断是否输入查询的关键字
    		sql.append("AND CONCAT(S.NAME,S.STA_NUM) LIKE '%"+keyName+"%'");
    	}
    	if(StringUtils.isNotEmpty(page.getStartTime())) {            //判断是否输入起始时间
    		sql.append("AND to_char(S.IMPL_DATE,'yyyy-MM-dd HH24:mi:ss')>='"+page.getStartTime()+"'");
    	}
    	if(StringUtils.isNotEmpty(page.getEndTime())) {            //判断是否输入结束时间
    		sql.append("AND to_char(S.IMPL_DATE,'yyyy-MM-dd HH24:mi:ss')<='"+page.getEndTime()+"'");
    	}
    	
    	Query query = entityManager.createNativeQuery(sql.toString());
    	Long count = Long.parseLong(query.getSingleResult().toString());
        return count;
    }
    
    /**发布历史查询
     * 条件查询记录并分页
     * @param page  页
     * @param type  查询类型
     * @param keyName  查询的关键字
     * @return
     */   
    public List<Map<String,Object>> findListByPageHistory(PageParam page,String type, String keyName){
    	Date date=new Date();
    	SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd");
    	String s=sf.format(date);//创建当前时间（字符串类型）
    	StringBuffer sql = new StringBuffer(" SELECT ROWNUM RN ,T.* FROM ( ");
    	sql.append("SELECT S.ID, S.STA_NUM, S.SAMPLING, S.NAME, S.TYPE_NUM, S.STATUS, to_char(S.IMPL_DATE,'yyyy-MM-dd') IMPL_DATE, S.URL,");
    	sql.append("S.DEL_NUM, to_char(S.DELETE_DATE,'yyyy-MM-dd HH24:mi:ss'), S.CREATOR, S.VIEW_COUNT, S.DOWNLOAD_COUNT ");
    	sql.append("FROM NERDS.STANDARD_MANAGE S WHERE S.DEL_NUM IS NULL AND S.IMPL_DATE <= SYSDATE ");   	
    	if(StringUtils.isNotEmpty(type)){      //判断是否输入类型
    		sql.append("AND S.TYPE_NUM LIKE '%"+type+"%'");   
    	}
    	if(StringUtils.isNotEmpty(keyName)) {    //判断是否输入查询的关键字
    		sql.append("AND CONCAT(S.NAME,S.STA_NUM) LIKE '%"+keyName+"%'");
    	}
    	if(StringUtils.isNotEmpty(page.getStartTime())) {            //判断是否输入起始时间
    		sql.append("AND to_char(S.IMPL_DATE,'yyyy-MM-dd HH24:mi:ss')>='"+page.getStartTime()+"'");
    	}
    	if(StringUtils.isNotEmpty(page.getEndTime())) {            //判断是否输入结束时间
    		sql.append("AND to_char(S.IMPL_DATE,'yyyy-MM-dd HH24:mi:ss')<='"+page.getEndTime()+"'");
    	}
    	sql.append("ORDER BY to_char(S.IMPL_DATE,'yyyy-MM-dd HH24:mi:ss') DESC");
    	sql.append(" ) T ");
    	 Query query = entityManager.createNativeQuery(sql.toString());
         query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
         query.setFirstResult(page.getOffset());
         query.setMaxResults(page.getLimit());
         List<Map<String, Object>> list = query.getResultList();
         return list;    	
    }
    /**
     * 增加新记录
     * @param workManageEntity
     * @return
     */
    public Map<String,Object> insertWork(StandardManageEntity standardManageEntity){
    	String idSql = "SELECT NERDS.NUMBER_SEQ.nextval from dual";
    	Query idQuery = entityManager.createNativeQuery(idSql);
        Integer id = Integer.parseInt(idQuery.getSingleResult().toString());
    	Map<String,Object> res = new HashMap<>();
    	String sampling = null;
    	if(standardManageEntity.getSampling().intValue()==0) {//是否采样：0-不采样  1-采样
    		sampling="否";
    	}else {
    		sampling="采";
    	}
    	StringBuffer sql = new StringBuffer();
    	sql.append("INSERT INTO NERDS.STANDARD_MANAGE (ID,STA_NUM,SAMPLING,NAME,TYPE_NUM,STATUS,IMPL_DATE,URL,DEL_NUM,CREATOR,VIEW_COUNT,DOWNLOAD_COUNT,CREATE_TIME) VALUES ");
    	sql.append("("+id+",");
    	sql.append("'"+standardManageEntity.getStaNum()+"',");
    	sql.append("'"+sampling+"',");
    	sql.append("'"+standardManageEntity.getName()+"',");
    	sql.append("'"+standardManageEntity.getTypeNum()+"',");
    	sql.append("'"+standardManageEntity.getStatus()+"',");
    	sql.append(""+"to_date ( '"+standardManageEntity.getImplDate()+"', 'YYYY-MM-DD HH24:MI:SS' )"+",");
    	sql.append("'"+standardManageEntity.getUrl()+"',");
    	sql.append(""+standardManageEntity.getDelNum()+",");
    	sql.append("'"+standardManageEntity.getCreator()+"',");
    	sql.append(""+standardManageEntity.getViewCount()+",");
    	sql.append(""+standardManageEntity.getDownloadCount()+",");
    	sql.append(" SYSDATE )");
    	
    	try {
            Query query = entityManager.createNativeQuery(sql.toString());

            
            if (query.executeUpdate() == 1){
                res.put("success", "1");
                res.put("msg", "保存成功");
            }else{
            	res.put("success", "0");
                res.put("msg", "保存失败");
            }
        } catch (Exception e) {
            res.put("success", "0");
            res.put("msg", e.toString());
            e.printStackTrace();
        }
        return res;
    }
    
    /**
     * 增加浏览数
* @param id
* @return
*/
public boolean changeViewCountById(Integer id) {
 String updateViewSql = " UPDATE NERDS.STANDARD_MANAGE SET VIEW_COUNT = (VIEW_COUNT+1) WHERE ID = ? ";
 Query updateQuery = entityManager.createNativeQuery(updateViewSql);
 updateQuery.setParameter(1, id);
 if(updateQuery.executeUpdate() == 1){
     return true;
 }
 return false;
}
/**
     * 增加下载数
* @param id
* @return
*/
public boolean changeDownloadCountById(Integer id) {
 String updateDownloadSql = " UPDATE NERDS.STANDARD_MANAGE SET DOWNLOAD_COUNT = (DOWNLOAD_COUNT+1) WHERE ID = ? ";
 Query updateQuery = entityManager.createNativeQuery(updateDownloadSql);
 updateQuery.setParameter(1, id);
 if(updateQuery.executeUpdate() == 1){
     return true;
 }
 return false;
}

/**
* 逻辑删除记录：DEL_NUM=1为删除，DEL_NUM=0为可使用
* @param id
* @return
*/
public boolean deleteWork(Integer id) {
	Date date=new Date();
	SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	String s=sf.format(date);//创建当前时间（字符串类型）
	StringBuffer deleteSql = new StringBuffer();
	deleteSql.append(" UPDATE NERDS.STANDARD_MANAGE SET DEL_NUM = '1',");
	deleteSql.append("DELETE_DATE="+"to_date ( '"+s+"', 'YYYY-MM-DD HH24:MI:SS' ) ");
	deleteSql.append(""+"WHERE ID = '"+id+"'");
	Query query = entityManager.createNativeQuery(deleteSql.toString());
 if(query.executeUpdate() == 1 ){
     return true;
 }
 return false;
}
/**   
 * @Title: getDate   
 * @Description: TODO(这里用一句话描述这个方法的作用)    
 * @param @return    设定文件      
 * @author ADC_ZH 
 * @date 下午4:07:36 
*/   
public Map<String, Object> getDate() {
	StringBuilder sql = new StringBuilder();
    sql.append(" SELECT to_char(CREATE_TIME,'yyyy-MM-dd')as CREATE_TIME FROM NERDS.STANDARD_MANAGE ORDER BY CREATE_TIME DESC ");
    SQLQuery query = entityManager.createNativeQuery(sql.toString())
            .unwrap(org.hibernate.SQLQuery.class);
    query.setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
    List<Map<String, Object>> list = query.list();
    Map<String, Object> map=list.get(0);
	return map;
}
/**   
 * @Title: adviseCount   
 * @Description: TODO(这里用一句话描述这个方法的作用)    
 * @param @param page
 * @param @param string
 * @param @param keyName
 * @param @return    设定文件   
 * @return Long    返回类型   
 * @author ADC_ZH 
 * @date 下午6:25:07 
*/   
public Long adviseCount(PageParam page, String type, String keyName) {
	StringBuffer sql = new StringBuffer();
	sql.append("SELECT COUNT(*) FROM NERDS.STANDARD_MANAGE S WHERE S.DEL_NUM IS NULL ");
	if(StringUtils.isNotEmpty(type)){      //判断是否输入类别
		sql.append("AND S.TYPE_NUM LIKE '%"+type+"%'");   
	}
	if(StringUtils.isNotEmpty(keyName)) {    //判断是否输入查询的关键字
		sql.append("AND CONCAT(S.NAME,S.STA_NUM) LIKE '%"+keyName+"%'");
	}
	if(StringUtils.isNotEmpty(page.getStartTime())) {            //判断是否输入起始时间
		sql.append("AND to_char(S.IMPL_DATE,'yyyy-MM-dd HH24:mi:ss')>='"+page.getStartTime()+"'");
	}
	if(StringUtils.isNotEmpty(page.getEndTime())) {            //判断是否输入结束时间
		sql.append("AND to_char(S.IMPL_DATE,'yyyy-MM-dd HH24:mi:ss')<='"+page.getEndTime()+"'");
	}
	
	Query query = entityManager.createNativeQuery(sql.toString());
	Long count = Long.parseLong(query.getSingleResult().toString());
    return count;
}
/**   
 * @Title: adviseList   
 * @Description: TODO(这里用一句话描述这个方法的作用)    
 * @param @param page
 * @param @param string
 * @param @param keyName
 * @param @return    设定文件   
 * @return List<Map<String,Object>>    返回类型   
 * @author ADC_ZH 
 * @date 下午6:25:13 
*/   
public List<Map<String, Object>> adviseList(PageParam page, String type,
		String keyName) {
	StringBuffer sql = new StringBuffer(" SELECT ROWNUM RN ,T.* FROM ( ");
	sql.append("SELECT S.ID, S.STA_NUM, S.SAMPLING, S.NAME, S.TYPE_NUM, S.STATUS, to_char(S.IMPL_DATE,'yyyy-MM-dd') IMPL_DATE, S.URL,");
	sql.append("S.DEL_NUM, to_char(S.DELETE_DATE,'yyyy-MM-dd HH24:mi:ss'), S.CREATOR, S.VIEW_COUNT, S.DOWNLOAD_COUNT ,");
	sql.append("  ( CASE WHEN SYSDATE-3<S.CREATE_TIME THEN '1' ELSE '0' END ) ISNEW  ");
	sql.append("FROM NERDS.STANDARD_MANAGE S WHERE S.DEL_NUM IS NULL  ");   	
	if(StringUtils.isNotEmpty(type)){      //判断是否输入类型
		sql.append("AND S.TYPE_NUM LIKE '%"+type+"%'");   
	}
	if(StringUtils.isNotEmpty(keyName)) {    //判断是否输入查询的关键字
		sql.append("AND CONCAT(S.NAME,S.STA_NUM) LIKE '%"+keyName+"%'");
	}  
	if(StringUtils.isNotEmpty(page.getStartTime())) {            //判断是否输入起始时间
		sql.append("AND to_char(S.IMPL_DATE,'yyyy-MM-dd HH24:mi:ss')>='"+page.getStartTime()+"'");
	}
	if(StringUtils.isNotEmpty(page.getEndTime())) {            //判断是否输入结束时间
		sql.append("AND to_char(S.IMPL_DATE,'yyyy-MM-dd HH24:mi:ss')<='"+page.getEndTime()+"'");
	}
	sql.append("ORDER BY IMPL_DATE DESC");
	sql.append(" ) T ");
	 Query query = entityManager.createNativeQuery(sql.toString());
     query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
     query.setFirstResult(page.getOffset());
     query.setMaxResults(page.getLimit());
     List<Map<String, Object>> list = query.getResultList();
     return list;    	
}
}
