package com.adc.da.bs.param.rest;

import com.adc.da.bs.param.entity.NerdsInfoEO;
import com.adc.da.bs.param.service.MbService;
import com.adc.da.bs.reportManage.entity.ReportManageEO;
import com.adc.da.util.http.Result;
import com.adc.da.util.utils.FileUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * HSSF 03版
 * XSSF 07版
 * SXSS 07版升级
 */

@RestController
@RequestMapping("/${restPath}/mb-test/")
@Api(tags = "mb-test", description = "模板相关功能")
public class ExcelWriteTest {
    private static final Logger logger = LoggerFactory.getLogger(ExcelWriteTest.class);

    //路径注意设置成自己的
    String PATH ="D:\\nerds\\";

    @Autowired
    private MbService mbService;

    @ApiOperation(value = "导入推荐目录发布数据", produces = "application/octet-stream")
    @RequestMapping(value = "/import", method = RequestMethod.POST)
    public void nerdsImportWrite(HttpServletResponse response,@RequestParam("file") MultipartFile file, String pc){
        List<ReportManageEO> reportEs = new ArrayList<ReportManageEO>();

        try {
            boolean fileFlg = false;
            String suffix="xlsx";
            List<MultipartFile> fileSearched = new ArrayList<> ();

            // 获取模板文件
            String fileExtension = FileUtil.getFileExtension(file.getOriginalFilename());
            // 读取模板文件
            File importFile = FileUtil.multipartFileToFile(file);
            if (importFile != null) {
                suffix = "." + suffix;
                File tempFile = File.createTempFile("temp", suffix);
                tempFile.deleteOnExit();
                FileUtil.copyFile(importFile, tempFile);
                Map<String, List<String>> data = importFromExcel(tempFile);

                List<NerdsInfoEO> result = new ArrayList<NerdsInfoEO>();
                Set<String> keySet = data.keySet();
                for (String key : keySet) {
                    if (!"".equals (key)) {
                        List<String> list = data.get (key);
                        NerdsInfoEO info = new NerdsInfoEO ();
//                        System.out.println ("list：" + list.toString ());
                        info.setPz_id (list.get (0));
                        info.setParamName (list.get (1).replace ("：", ""));
                        info.setParamValue (list.get (2));
                        info.setQymc (list.get (3));
                        info.setCpsb (list.get (4));
                        info.setCpxh (list.get (5));
                        info.setCpmc (list.get (6));
                        info.setxPc (list.get (7).replace("（", "").replace ("）", "" ));
                        info.setPc (pc);

                        result.add (info);
                    }
                }
                // 导入数据
                mbService.importExcel(result);
            }
        } catch(Exception e){
            Result.error("批量上传失败，请联系管理员.");
            logger.error(e.getMessage(), e);
        }
    }

    public static Map<String, List<String>> importFromExcel(File srcFile) throws Exception {
        Map<String, List<String>> resFormExcel = new HashMap<String, List<String>> ();
        FileInputStream fis = new FileInputStream(srcFile);
        try {
            if (srcFile.getName().endsWith(".xls")) {
                POIFSFileSystem fSystem = new POIFSFileSystem (fis);
                HSSFWorkbook wb = new HSSFWorkbook (fSystem);
                HSSFSheet st = wb.getSheetAt (0);

                int excelRealRow = getExcelRealRow (st);
                for (int i = 0; i < 1 + excelRealRow; i++) {
                    String key = getValueAt (st, i, 0);
                    List<String> lineData = new ArrayList<String> ();
                    for (int j = 1; j < 9; j++) {
                        lineData.add (getValueAt (st, i, j));
                    }
                    resFormExcel.put (key, lineData);
                }
                return resFormExcel;
            }
            else if (srcFile.getName().endsWith(".xlsx")) {
                XSSFWorkbook xwb = new XSSFWorkbook(fis);
                XSSFSheet xst = xwb.getSheetAt(0);

                int excelRealRow = getExcelRealRow (xst);
                for (int i = 0; i < 1 + excelRealRow; i++) {
                    String key = getValueAt (xst, i, 0);
                    List<String> lineData = new ArrayList<String> ();
                    for (int j = 1; j < 9; j++) {
                        lineData.add (getValueAt (xst, i, j));
                    }
                    resFormExcel.put (key, lineData);
                }
                return resFormExcel;
            } else {
                throw new Exception("文件类型不对!");
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            throw new FileNotFoundException(e.getMessage());
        } catch (IOException e) {
            e.printStackTrace();
            throw new Exception(e.getMessage());
        }
        finally{
            fis.close();
        }
    }

    static // 获取Excel表的真实行数
    public int getExcelRealRow(Sheet sheet) {
        boolean flag = false;
        for (int i = 1; i <= sheet.getLastRowNum(); ) {
            Row r = sheet.getRow(i);
            if (r == null) {
                // 如果是空行（即没有任何数据、格式），直接把它以下的数据往上移动
                sheet.shiftRows(i + 1, sheet.getLastRowNum(), -1);
                continue;
            }
            flag = false;
            for (Cell c : r) {
                if (c.getCellType() != Cell.CELL_TYPE_BLANK) {
                    flag = true;
                    break;
                }
            }
            if (flag) {
                i++;
                continue;
            } else {
                // 如果是空白行（即可能没有数据，但是有一定格式）
                if (i == sheet.getLastRowNum())// 如果到了最后一行，直接将那一行remove掉
                    sheet.removeRow(r);
                else//如果还没到最后一行，则数据往上移一行
                    sheet.shiftRows(i + 1, sheet.getLastRowNum(), -1);
            }
        }
        return sheet.getLastRowNum();
    }

    public static String getValueAt(Object sheet , int row, int col){
        String res_rString = "";
        if (sheet instanceof HSSFSheet) {
            HSSFCell sCell = ((HSSFSheet) sheet).getRow(row).getCell((short)col);

//            System.out.println(row);
//            System.out.println(col);
//            System.out.println ("sCell.getCellType():"+sCell.getCellType());

            // 判断单元格数据是否是日期
            if ("yyyy/mm;@".equals(sCell.getCellStyle().getDataFormatString())
                    || "m/d/yy".equals(sCell.getCellStyle().getDataFormatString())
                    || "yy/m/d".equals(sCell.getCellStyle().getDataFormatString())
                    || "mm/dd/yy".equals(sCell.getCellStyle().getDataFormatString())
                    || "dd-mmm-yy".equals(sCell.getCellStyle().getDataFormatString())
                    || "yyyy/m/d".equals(sCell.getCellStyle().getDataFormatString())) {

                res_rString = importByExcelForDate(sCell);
                return res_rString;
            }

            int cellT = sCell.getCellType();
            switch (cellT) {
                case HSSFCell.CELL_TYPE_NUMERIC:
                    res_rString = StringUtils.defaultString(String.valueOf(sCell.getNumericCellValue()));
                    Double parseDouble = Double.parseDouble(res_rString);
                    int intValue = parseDouble.intValue();
                    res_rString =intValue+"";
                    break;
                case HSSFCell.CELL_TYPE_STRING:
                    res_rString = StringUtils.defaultString(String.valueOf(sCell.getStringCellValue()));
                    break;
                default:
                    return "";
            }
            return res_rString;
        }
        else if (sheet instanceof XSSFSheet) {
            XSSFCell xCell = ((XSSFSheet) sheet).getRow(row).getCell((short)col);

            if(xCell ==null) {
                return "";
            }
//            System.out.println("sCell.row():"+row);
//            System.out.println("sCell.col():"+col);
//            System.out.println ("sCell.getCellType():"+xCell.getCellType());
            xCell.setCellType(Cell.CELL_TYPE_STRING);
            System.out.println ("sCell.getStringCellValue():"+xCell.getStringCellValue());

            res_rString = StringUtils.defaultString(String.valueOf(xCell.getStringCellValue()));

/*            // 判断单元格数据是否是日期
            if ("yyyy/mm;@".equals(xCell.getCellStyle().getDataFormatString())
                    || "m/d/yy".equals(xCell.getCellStyle().getDataFormatString())
                    || "yy/m/d".equals(xCell.getCellStyle().getDataFormatString())
                    || "mm/dd/yy".equals(xCell.getCellStyle().getDataFormatString())
                    || "dd-mmm-yy".equals(xCell.getCellStyle().getDataFormatString())
                    || "yyyy/m/d".equals(xCell.getCellStyle().getDataFormatString())) {

                res_rString = importByExcelForDate(xCell);
                return res_rString;
            }*/

            /*int cellT = xCell.getCellType();
            switch (cellT) {
            case HSSFCell.CELL_TYPE_NUMERIC:
                res_rString = StringUtils.defaultString(String.valueOf(xCell.getNumericCellValue()));
                Double parseDouble = Double.parseDouble(res_rString);
                int intValue = parseDouble.intValue();
                res_rString =intValue+"";
                break;
            case HSSFCell.CELL_TYPE_STRING:
                res_rString = StringUtils.defaultString(String.valueOf(xCell.getStringCellValue()));
                break;
            default:
                return "";
            }*/
            return res_rString;
        }
        else {
            return "";
        }
    }

    public static String importByExcelForDate(Cell currentCell) {
        String currentCellValue = "";
        // 判断单元格数据是否是日期
        if ("yyyy/mm;@".equals(currentCell.getCellStyle().getDataFormatString())
                || "m/d/yy".equals(currentCell.getCellStyle().getDataFormatString())
                || "yy/m/d".equals(currentCell.getCellStyle().getDataFormatString())
                || "mm/dd/yy".equals(currentCell.getCellStyle().getDataFormatString())
                || "dd-mmm-yy".equals(currentCell.getCellStyle().getDataFormatString())
                || "yyyy/m/d".equals(currentCell.getCellStyle().getDataFormatString())) {
            if (DateUtil.isCellDateFormatted(currentCell)) {
                // 用于转化为日期格式
                Date d = currentCell.getDateCellValue();
                DateFormat formater = new SimpleDateFormat ("yyyy-MM-dd");
                currentCellValue = formater.format(d);
            }
        } else {
            // 不是日期原值返回
            currentCellValue = currentCell.toString();
        }
        return currentCellValue;
    }

    /**
     * 07版本
     */
    @ApiOperation(value = "按模板导出表格数据", produces = "application/octet-stream")
    @RequestMapping(value = "/export", method = RequestMethod.POST)
    public void testWrite07(HttpServletResponse response){
        //1、创建一个工作簿 07
        Workbook workbook = new XSSFWorkbook();
        //2、创建一个工作表
        Sheet sheet = workbook.createSheet("CanPOI练习表07");
        //3、创建一个行  (1,1)
        Row row_1 = sheet.createRow(0); //0代表第一行
        //4、创建一个单元格
        Cell cell_1_1 = row_1.createCell(0);//0代表第一列
        cell_1_1.setCellValue("大家加油呀"); //第一行第一列插入数据
        Cell cell_1_2 = row_1.createCell(1);//1代表第二列
        cell_1_2.setCellValue(666);//第一行的第二列插入数据

        //第二行
        Row row_2 = sheet.createRow(1); //创建第二行
        //(2,1)
        Cell cell_2_1 = row_2.createCell(0); //创建第二行第一列
        cell_2_1.setCellValue("第二行第一列");
        Cell cell_2_2 = row_2.createCell(1); //第二行第二列
        String date = new DateTime().toString("yyyy-MM-dd HH:mm:ss");
        cell_2_2.setCellValue(date);

        //生产一张表(IO流)
        //07版本就是使用 xlsx结尾！
        try(FileOutputStream fileOutputStream = new FileOutputStream(PATH + "CanPOI练习表07版本.xlsx");) {
            //输出
            workbook.write(fileOutputStream);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        System.out.println("CanPOI练习表03版本.xlsx 生产完毕！");
    }
}