package com.adc.da.bs.report.rest;

import com.adc.da.bs.report.service.AnalyzeReportService;
import com.adc.da.util.http.ResponseMessage;
import com.adc.da.util.http.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

/**
 * 分析报告页面
 *
 * @author 韩磊
 * @date 2019/3/2:54
 */
@RestController
@RequestMapping(value = "/${restPath}/AnalyzeReport")
@Api(description = "分析报告页面")
public class AnalyzeReportController {

    @Autowired
    private AnalyzeReportService analyzeReportService;

    /**
     * @describtion 分析报告-公告
     * @author AnYuShi
     * @modified by
     */
    @ApiOperation(value = "分析报告-公告首页显示")
    @GetMapping("/analyzeReportIndex")
    @ResponseBody
    public ResponseMessage<Map<String, Object>> reportIndex() {
       try {
           Map<String, Object> maps = analyzeReportService.analyzeReportIndex();
           return Result.success(maps);
       }catch (Exception e){
           return Result.error(e.getMessage());

       }
    }

    /**
     * @describtion 分析报告-公告/新能源
     * @author AnYuShi
     * @modified by
     */
    @ApiOperation(value = "分析报告-新能源首页显示")
    @GetMapping("/analyzeNewEnergyIndex")
    @ResponseBody
    public ResponseMessage<Map<String, Object>> newEnergyIndex() {
       try {
           Map<String, Object> maps = analyzeReportService.analyzeNewEnergyIndex();
           return Result.success(maps);
       }
        catch (Exception e){
            return Result.error(e.getMessage());

        }
    }

    /**
     * @describtion 分析报告-公告/新能源列表
     * @author an
     * @modified by
     */
    @ApiOperation(value = "公告检测报告查询列表")
    @GetMapping("/analyzeReportList")
    @ResponseBody
    public ResponseMessage<Map<String, Object>> reportList() {
       try {
           Map<String, Object> maps = analyzeReportService.analyzeReportList();
           return Result.success(maps);
       }
        catch (Exception e){
            return Result.error(e.getMessage());

        }
    }

    /**
     * @describtion 分析报告-公告/新能源列表
     * @author an
     * @modified by
     */
    @ApiOperation(value = "新能源检测报告查询列表")
    @GetMapping("/analyzeNewEnergyList")
    @ResponseBody
    public ResponseMessage<Map<String, Object>> newEnergyList() {
        try {
            Map<String, Object> maps = analyzeReportService.analyzeNewEnergyList();
            return Result.success(maps);
        }
        catch (Exception e){
            return Result.error(e.getMessage());

        }
    }

}
