package com.adc.da.bs.reportManage.dao;

import com.adc.da.bs.reportManage.entity.ReportManageVO;
import com.adc.da.bs.reportManage.entity.ReportUploadVO;
import com.adc.da.bs.reportManage.utils.PoiUtil;
import com.adc.da.util.exception.AdcDaBaseException;
import com.adc.da.util.http.PageParam;

import org.apache.log4j.Logger;
import org.hibernate.transform.Transformers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import java.io.File;
import java.util.List;
import java.util.Map;

/**
 * @ProjectName: ADCCVT
 * @Package: com.adc.cvt.FileManage.dao
 * @ClassName: FileManageDao
 * @Author: deng
 * @Description: 文件管理dao
 * @Date: 2020/9/25 10:52
 * @Version: 1.0
 */
@Component
public class ReportManageDao {
    private static Logger log = Logger.getLogger(ReportManageDao.class);

    protected final EntityManager entityManager;

    @Autowired
    public ReportManageDao(EntityManager entityManager) {
        super();
        this.entityManager = entityManager;
    }

    /**
     * @param page
     * @Method countAll
     * @Author deng
     * @Version 1.0
     * @Description 根据条件查询总数
     * @Return int
     * @Exception
     * @Date 2020/9/25 11:04
     */
    public Long findTotalCount(PageParam page, ReportManageVO reportManageVO) {

        StringBuilder sql = new StringBuilder("SELECT count(*) FROM GGCAIN06.BGK where qymc = '" + reportManageVO.getQymc() + "' ");
        // 检索条件
        String mulSearchName = reportManageVO.getMulSearchName();
        String cpsb = reportManageVO.getCpsb();
        String bgzl = reportManageVO.getReportType();
        String confirm = reportManageVO.getConfirm();
        String confirmTimeB =reportManageVO.getConfirmTimeB();
        String confirmTimeE =reportManageVO.getConfirmTimeE();

        // 报告编号等条件  origin （0：检测报告查询界面 | 1：检测报告下载界面）
        if (!StringUtils.isEmpty(mulSearchName) && ("0".equals(reportManageVO.getOrigin()) || reportManageVO.getOrigin() == null)) {
            sql.append(" and (CPXH like concat(concat('%',:mulSearchName),'%') or BGBH like concat(concat('%',:mulSearchName),'%') or XMMC like concat(concat('%',:mulSearchName),'%')) ");
        } else if (!StringUtils.isEmpty(mulSearchName) && "1".equals(reportManageVO.getOrigin())){
            sql.append(" and (CPXH like concat(concat('%',:mulSearchName),'%') or BGBH like concat(concat('%',:mulSearchName),'%') or XMMC like concat(concat('%',:mulSearchName),'%') or");
            sql.append(" CPMC like concat(concat('%', :mulSearchName), '%') or CPSB like concat(concat('%', :mulSearchName), '%')) ");
        }

        if (("0".equals(reportManageVO.getOrigin()) || reportManageVO.getOrigin() == null)) {
            sql.append(" and jgdh is null");
        }

        // 选择状态(Y;N)
        if (!StringUtils.isEmpty(confirm)) {
            sql.append(" and confirm = :confirm");
        }

        // 商标
        if (!StringUtils.isEmpty(cpsb)) {
            sql.append(" and cpsb like concat(concat('%',:cpsb),'%')");
        }

        // 报告类型
        if (!StringUtils.isEmpty(bgzl)) {
            sql.append(" and bgzl = :bgzl");
        }

        // 确认开始时间
        if (confirmTimeB != null && !confirmTimeB.isEmpty()) {
            sql.append(" and TO_CHAR(CONFIRM_TIME, 'YYYY-MM-DD') >= :confirmTimeB");
        }

        // 确认结束时间
        if (confirmTimeE != null && !confirmTimeE.isEmpty()) {
            sql.append(" and TO_CHAR(CONFIRM_TIME, 'YYYY-MM-DD') <= :confirmTimeE");
        }

        Query query = entityManager.createNativeQuery(sql.toString());
        if (!StringUtils.isEmpty(mulSearchName)) {
            query.setParameter("mulSearchName", mulSearchName);
        }
        if (!StringUtils.isEmpty(cpsb)) {
            query.setParameter("cpsb", cpsb);
        }
        if (!StringUtils.isEmpty(bgzl)) {
            query.setParameter("bgzl", bgzl);
        }

        if (!StringUtils.isEmpty(confirm)) {
            query.setParameter("confirm", confirm);
        }

        if (!StringUtils.isEmpty(confirmTimeB)) {
            query.setParameter("confirmTimeB", confirmTimeB);
        }

        if (!StringUtils.isEmpty(confirmTimeE)) {
            query.setParameter("confirmTimeE", confirmTimeE);
        }

        Long count = Long.parseLong(query.getSingleResult().toString());
        return count;
    }

    public List<Map<String, Object>> queryListByPage(PageParam page, ReportManageVO reportManageVO) {
        StringBuilder sql = new StringBuilder("");
        sql.append(" SELECT a.*, ");
        sql.append("        case ");
        sql.append("          when a.jgdh is not null then ");
        sql.append("           '" + PoiUtil.BGBHPATHOLD + "' || b.username || '/' || a.bgbh ");
        sql.append("          else ");
        sql.append("           '" + PoiUtil.BGBHPATH + "' || a.bgbh ");
        sql.append("        end as reportPath, ");
        sql.append("        case when confirm_time is null then 'N/A' else TO_CHAR(confirm_time, 'YYYY-MM-DD') end as confirm_date ");
        sql.append("   FROM GGCAIN06.BGK a ");
        sql.append("   left join ggcain06.jcejg b ");
        sql.append("     on substr(a.bgbh, 0, 2) = b.jgdh ");
        sql.append("  where a.qymc = '" + reportManageVO.getQymc() + "' ");

        // 检索条件
        String mulSearchName = reportManageVO.getMulSearchName();
        String cpsb = reportManageVO.getCpsb();
        String bgzl = reportManageVO.getReportType();
        String confirm = reportManageVO.getConfirm();
        String confirmTimeB =reportManageVO.getConfirmTimeB();
        String confirmTimeE =reportManageVO.getConfirmTimeE();

        // 报告编号等条件  origin （0：检测报告查询界面 | 1：检测报告下载界面）
        if (!StringUtils.isEmpty(mulSearchName) && ("0".equals(reportManageVO.getOrigin()) || reportManageVO.getOrigin() == null)) {
            sql.append(" and (CPXH like concat(concat('%',:mulSearchName),'%') or BGBH like concat(concat('%',:mulSearchName),'%') or XMMC like concat(concat('%',:mulSearchName),'%')) ");
        } else if (!StringUtils.isEmpty(mulSearchName) && "1".equals(reportManageVO.getOrigin())){
            sql.append(" and (CPXH like concat(concat('%',:mulSearchName),'%') or BGBH like concat(concat('%',:mulSearchName),'%') or XMMC like concat(concat('%',:mulSearchName),'%') or");
            sql.append(" CPMC like concat(concat('%', :mulSearchName), '%') or CPSB like concat(concat('%', :mulSearchName), '%')) ");
        }

        if (("0".equals(reportManageVO.getOrigin()) || reportManageVO.getOrigin() == null)) {
            sql.append(" and a.jgdh is null");
        } else {
            // 默认追加入库状态查询条件 add by deng 20210409
            sql.append(" and a.rk='Y'");
        }

        // 选择状态(Y;N)
        if (!StringUtils.isEmpty(confirm)) {
            sql.append(" and confirm = :confirm");
        }

        // 商标
        if (!StringUtils.isEmpty(cpsb)) {
            sql.append(" and cpsb like concat(concat('%',:cpsb),'%')");
        }

        // 报告类型
        if (!StringUtils.isEmpty(bgzl)) {
            sql.append(" and bgzl = :bgzl");
        }

        // 确认开始时间
        if (confirmTimeB != null && !confirmTimeB.isEmpty()) {
            sql.append(" and TO_CHAR(CONFIRM_TIME, 'YYYY-MM-DD') >= :confirmTimeB");
        }

        // 确认结束时间
        if (confirmTimeE != null && !confirmTimeE.isEmpty()) {
            sql.append(" and TO_CHAR(CONFIRM_TIME, 'YYYY-MM-DD') <= :confirmTimeE");
        }

        sql.append(" ORDER BY WJ_TIME DESC ");

        Query query = entityManager.createNativeQuery(sql.toString());
        if (!StringUtils.isEmpty(mulSearchName)) {
            query.setParameter("mulSearchName", mulSearchName);
        }
        if (!StringUtils.isEmpty(cpsb)) {
            query.setParameter("cpsb", cpsb);
        }
        if (!StringUtils.isEmpty(bgzl)) {
            query.setParameter("bgzl", bgzl);
        }

        if (!StringUtils.isEmpty(confirm)) {
            query.setParameter("confirm", confirm);
        }

        if (!StringUtils.isEmpty(confirmTimeB)) {
            query.setParameter("confirmTimeB", confirmTimeB);
        }

        if (!StringUtils.isEmpty(confirmTimeE)) {
            query.setParameter("confirmTimeE", confirmTimeE);
        }

        log.info("******findreportlist" + sql.toString());

        query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
        query.setFirstResult(page.getOffset());
        query.setMaxResults(page.getLimit());
        List<Map<String, Object>> list = query.getResultList();
        return list;
    }

    /**
     * 保存上传报告文件信息
     *
     * @param reportUploadVO
     */
    public int saveReportDetail(ReportUploadVO reportUploadVO) {
        // 删除历史存在报告目录
        int k = deleteByBgbh(reportUploadVO.getBgbh(),reportUploadVO.getQymc());
        System.out.println("k"+k);
        int i=0;
        try {
            String inFileSql =
                    " INSERT INTO GGCAIN06.BGK (bgbh,xmdm,xmmc,upload_jcejgid,cpxh,cpmc,cpsb,bgzl,qymc,ml_time,wj_time,confirm)" +
                            " VALUES (?,?,?,?,?,?,?,?,?,SYSDATE,SYSDATE,'N') ";
            Query inFileQuery = entityManager.createNativeQuery(inFileSql);
            inFileQuery.setParameter(1, reportUploadVO.getBgbh());
            inFileQuery.setParameter(2, reportUploadVO.getBgbh().substring(5,7));
            inFileQuery.setParameter(3, reportUploadVO.getXmmc());
            inFileQuery.setParameter(4, reportUploadVO.getQyid());
            inFileQuery.setParameter(5, reportUploadVO.getCpxh());
            inFileQuery.setParameter(6, reportUploadVO.getCpmc());
            inFileQuery.setParameter(7, reportUploadVO.getCpsb());
            inFileQuery.setParameter(8, reportUploadVO.getReportType());
            inFileQuery.setParameter(9, reportUploadVO.getQymc());

            i = inFileQuery.executeUpdate();

        } catch (Exception e) {
            log.error(e.toString());
            throw new AdcDaBaseException("报告存储插入失败：" + reportUploadVO.getBgbh());
        }

        return i;
    }

    /**
     * @param bgbh
     * @Method deleteByBgbh
     * @Author deng
     * @Version 1.0
     * @Description 删除
     * @Return void
     * @Exception
     * @Date 2020/10/27 13:49
     */
    public int deleteByBgbh(String bgbh, String qymc) {
        String sql = "DELETE FROM GGCAIN06.BGK WHERE BGBH = ? AND CONFIRM ='N' AND QYMC = ? ";
        Query query = entityManager.createNativeQuery(sql.toString());
        query.setParameter(1,bgbh);
        query.setParameter(2,qymc);
        return query.executeUpdate();
    }


    public int updateConfirmByBgbh(String bgbh, String qymc) {
        String sql = "UPDATE GGCAIN06.BGK SET CONFIRM='Y', CONFIRM_TIME = sysdate WHERE BGBH = ? AND CONFIRM ='N' AND QYMC = ? ";
        Query query = entityManager.createNativeQuery(sql.toString());
        query.setParameter(1,bgbh);
        query.setParameter(2,qymc);
        return query.executeUpdate();
    }

    /**
     * 查询报告状态信息
     *
     * @param bgbh
     * @return
     */
    public List<String>  checkBgkStatus(String bgbh) {
        String sql = "SELECT CONFIRM FROM GGCAIN06.BGK WHERE BGBH=?";
        Query query = entityManager.createNativeQuery(sql.toString());
        query.setParameter(1,bgbh);

        return query.getResultList();
    }

    /**
     * 验证报告是否存在,如果存在返回其路径
     *
     * @param bgbh
     * @param reportPath
     * @return
     */
    public String searchJceBg(String bgbh, String reportPath) {
        String reportPath1 = reportPath.toString() + ".pdf";
        String reportPath2 = reportPath.toString() + ".PDF";
        String reportPath3 = reportPath.toString() + ".doc";
        String reportPath4 = reportPath.toString() + ".DOC";
        String reportPath5 = reportPath.toString() + ".docx";


        StringBuffer url = new StringBuffer().append("http://123.127.164.35:18082/datafilesAdd_8/");
        if(reportPath.contains("ADC")){
            url = new StringBuffer().append("http://www.catarc.info:13030/ADC/upload/report/");

            File file1 = new File(reportPath1);
            File file2 = new File(reportPath2);
            File file3 = new File(reportPath3);
            File file4 = new File(reportPath4);
            File file5 = new File(reportPath5);

            if (file1.exists()) {
                url.append(bgbh).append(".pdf");
                return url.toString();
            } else if (file2.exists()) {
                url.append(bgbh).append(".PDF");
                return url.toString();
            }  else if (file3.exists()) {
                url.append(bgbh).append(".doc");
                return url.toString();
            } else if (file4.exists()) {
                url.append(bgbh).append(".DOC");
                return url.toString();
            } else if (file5.exists()) {
                url.append(bgbh).append(".docx");
                return url.toString();
            }
        } else {
            // 公告服务器报告无法查看，基本上都是pdf文件，先按pdf设置
            url.append(bgbh).append(".pdf");
            return url.toString();
        }
        return null;
    }

    public Long findBgCount(String qymc, String bgbh) {

        StringBuilder sql = new StringBuilder("SELECT count(*) FROM GGCAIN06.BGK where qymc = :qymc and bgbh = :bgbh ");

        Query query = entityManager.createNativeQuery(sql.toString());
        if (!StringUtils.isEmpty(qymc)) {
            query.setParameter("qymc", qymc);
        }
        if (!StringUtils.isEmpty(bgbh)) {
            query.setParameter("bgbh", bgbh);
        }

        Long count = Long.parseLong(query.getSingleResult().toString());
        return count;
    }

	/**   
	 * @Title: getBgPath   
	 * @Description: TODO(这里用一句话描述这个方法的作用)    
	 * @param @param bgbh
	 * @param @return    设定文件   
	 * @return List<Map<String,Object>>    返回类型   
	 * @author ADC_ZH 
	 * @date 上午11:47:10 
	*/   
	public List<Map<String, Object>> getBgPath(String bgbh) {
        StringBuilder sql = new StringBuilder("");
        sql.append(" SELECT b.JGDH,case  when a.jgdh is not null ");
        sql.append(" then '" + PoiUtil.BGBHPATHOLD + "' || b.username || '/' || a.bgbh ");
        sql.append(" else '" + PoiUtil.BGBHPATH + "'|| a.bgbh ");
        sql.append(" end as BGPATH ");
        sql.append(" FROM GGCAIN06.BGK a ");
        sql.append(" left join ggcain06.jcejg b on substr(a.bgbh, 0, 2) = b.jgdh ");
        sql.append(" where BGBH =:bgbh ");
        Query query = entityManager.createNativeQuery(sql.toString());
        query.setParameter("bgbh", bgbh);
        query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP); 
        List<Map<String, Object>> list = query.getResultList();
        return list;
	}
}
