package com.adc.da.bs.reportManage.rest;

import com.adc.da.bs.reportManage.entity.ReportManageEO;
import com.adc.da.bs.reportManage.entity.ReportManageVO;
import com.adc.da.bs.reportManage.entity.ReportUploadVO;
import com.adc.da.bs.reportManage.service.ReportManageService;
import com.adc.da.bs.reportManage.utils.PoiUtil;
import com.adc.da.bs.reportManage.utils.ZipUtil;
import com.adc.da.util.exception.AdcDaBaseException;
import com.adc.da.util.http.PageParam;
import com.adc.da.util.http.ResponseMessage;
import com.adc.da.util.http.Result;
import com.adc.da.util.utils.FileUtil;
import com.adc.da.util.utils.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.io.FileUtils;
import org.dom4j.DocumentException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLDecoder;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * @ProjectName: ADCBS
 * @ClassName: ReportManageController
 * @Author: deng
 * @Description: 检测报告web
 * @Date: 2020/7/23 15:47
 * @Version: 1.0
 */
@RestController
@RequestMapping("/${restPath}/report_manage/")
@Api(tags = "report-manage-controller", description = "检测报告相关")
public class ReportManageController {

    private static final Logger logger = LoggerFactory.getLogger(ReportManageController.class);

    @Autowired
    private ReportManageService reportManageService;

    @ApiOperation(value="检测报告分页查询")
    @PostMapping("queryListByPage")
    @ResponseBody
    public Map<String, Object> queryListByPage(HttpServletRequest request, PageParam page, String confirm,String confirmTimeB,String confirmTimeE,
                                               String cpsb, String mulSearchName, String reportType, String origin) {
        Map<String,Object> usermap=(Map<String, Object>) request.getSession().getAttribute("usermap");
        String qyid=usermap.get("USERNAME").toString();
        String qymc=usermap.get("NAME").toString();
        ReportManageVO reportManageVO = new ReportManageVO();
        reportManageVO.setQyid(qyid);
        reportManageVO.setQymc(qymc);
        reportManageVO.setConfirm(confirm);
        reportManageVO.setConfirmTimeB(confirmTimeB);
        reportManageVO.setConfirmTimeE(confirmTimeE);
        reportManageVO.setCpsb(cpsb);
        reportManageVO.setMulSearchName(mulSearchName);
        reportManageVO.setReportType(reportType);
        reportManageVO.setOrigin(origin);

        Map<String,Object> pageMap =reportManageService.queryListByPage(page, reportManageVO);
        return pageMap;
    }

    /**
     * 上传文件
     *
     * @param request
     * @param reportUploadVO
     * @return java.util.Map<java.lang.String                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               ,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               java.lang.Object>
     * @author Deng
     * @date 2019/3/20 16:37
     */
    @SuppressWarnings("unchecked")
    @ApiOperation(value = "|ReportUploadVO|报告上传")
    @PostMapping("/uploadReport")
    public ResponseMessage uploadReport(HttpServletRequest request, @RequestParam("file") MultipartFile file, ReportUploadVO reportUploadVO) {
        Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");

        if (null == usermap || usermap.isEmpty()) {
            return Result.error("请先登录系统后再进行申请.");
        }

        reportUploadVO.setQyid(usermap.get("USERNAME").toString());
        reportUploadVO.setQymc(usermap.get("NAME").toString());

        if (null == file && StringUtils.isBlank(file.getOriginalFilename())) {
            return Result.error("请上传文件.");
        }

        String suffix = FileUtil.getFileExtension(file.getOriginalFilename());
        if (!("pdf".equals(suffix) || "PDF".equals(suffix) || "docx".equals(suffix) || "doc".equals(suffix) || "DOC".equals(suffix))) {
            return Result.error("上传的文件格式不正确.");
        }

        if (null == reportUploadVO.getReportType() || (!"ZCBG".equals(reportUploadVO.getReportType()) && !"LBJBG".equals(reportUploadVO.getReportType()))) {
            return Result.error("请重新选择报告文件类型.");
        }

        if (null == reportUploadVO.getBgbh() || reportUploadVO.getBgbh().length() != 13){
            return Result.error("报告目录中的报告编号必须为13位.");
        }

        if(FileUtil.getFileName(file.getOriginalFilename()).length() != 13){
            return Result.error("报告上传中的报告编号必须为13位.");
        }

        // 确认是否可以上传报告 如果已确认的报告不允许上传覆盖和更新数据
        List<String> reuslt = reportManageService.checkBgkStatus(reportUploadVO.getBgbh());
        if (reuslt.size()>0 && "Y".equals(reuslt.get(0).toString())) {
            return Result.error("该报告已存在并确认，无法覆盖和更新报告数据.");
        }
        int n = reportManageService.uploadFile(file, reportUploadVO, PoiUtil.ADDRESS);
        if (0 != n) {
            return Result.success();
        } else {
            return Result.error("系统忙，请联系管理员.");
        }
    }

    /**
     * 批量上传文件
     *
     * @param request
     * @param files
     * @return java.util.Map<java.lang.String                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               ,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               java.lang.Object>
     * @author Deng
     * @date 2019/3/20 16:37
     */
    @SuppressWarnings("unchecked")
    @ApiOperation(value = "|Reports|批量报告上传")
    @PostMapping(value = "/uploadReports",headers = "content-type=multipart/form-data")
//    public ResponseMessage<List<ReportManageEO>> uploadReports(HttpServletRequest request, @RequestParam("file") MultipartFile[] files) {
        public ResponseMessage uploadReports(HttpServletRequest request, @RequestParam("file") MultipartFile[] files) {
        Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");

        if (null == usermap || usermap.isEmpty()) {
            return Result.error("请先登录系统后再进行申请.");
        }

        StringBuffer reportSuccess = new StringBuffer("成功上传报告号：<br>");
        StringBuffer reportFailure = new StringBuffer("");

        String qyid=usermap.get("USERNAME").toString();
        String qymc=usermap.get("NAME").toString();

        if (null == files || files.length==0) {
            return Result.error("请选择需要批量上传检测报告的模板文件.");
        }
        List<ReportManageEO> reportEs = new ArrayList<ReportManageEO>();

        try {
            boolean fileFlg = false;
            String suffix="";
            List<MultipartFile> fileSearched = new ArrayList<>();

            // 获取模板文件
            for (MultipartFile file : files) {
                String fileExtension = FileUtil.getFileExtension(file.getOriginalFilename());
                if ("xls".equals(fileExtension) || "xlsx".equals(fileExtension)) {
                    fileFlg = true;
                    fileSearched.add(file);
                    suffix = fileExtension;
                    break;
                }
            }
            if (fileSearched.size()==0) {
                return Result.error("请使用模板文件上传批量报告.");
            }
            // 读取模板文件
            File importFile = FileUtil.multipartFileToFile(fileSearched.get(0));
            if (importFile != null) {
                suffix = "." + suffix;
                File tempFile = File.createTempFile("temp", suffix);
                tempFile.deleteOnExit();
                FileUtil.copyFile(importFile, tempFile);

                Map<String, List<String>> data = PoiUtil.importFromExcel(tempFile);

                Set<String> keySet = data.keySet();
                for (String key : keySet) {
                    if (!"".equals(key)) {
                        List<String> list = data.get(key);
                        ReportUploadVO reportUploadVO = new ReportUploadVO();
                        System.out.println("list：" + list.toString());
                        reportUploadVO.setBgbh(list.get(0));
                        reportUploadVO.setCpmc(list.get(1));
                        reportUploadVO.setCpxh(list.get(2));
                        reportUploadVO.setCpsb(list.get(3));
                        reportUploadVO.setXmmc(list.get(4));
                        String tmpReportType=list.get(5);
                        // 报告类型编辑
                        if ("整车".equals(tmpReportType)){
                            tmpReportType="ZCBG";
                        } else if("零部件".equals(tmpReportType)){
                            tmpReportType="LBJBG";
                        } else {
                            tmpReportType="";
                        }

                        reportUploadVO.setReportType(tmpReportType);
                        reportUploadVO.setQyid(qyid);
                        reportUploadVO.setQymc(qymc);

                        String currentReportName=list.get(6);
                        System.out.println("文件上传路径：" + PoiUtil.ADDRESS + list.get(6));
                        boolean findReport = false;
                        //获取对应报告文件
                        for (MultipartFile file : files) {
                            if(currentReportName.equals(FileUtil.getFileName(file.getOriginalFilename()))){
                                findReport = true;
                                List<String> reuslt = reportManageService.checkBgkStatus(reportUploadVO.getBgbh());
                                if (reuslt.size()>0 && "Y".equals(reuslt.get(0).toString())) {
                                    reportFailure.append ("报告已存在并确认，无法覆盖和更新报告数据，报告号：").append (reportUploadVO.getBgbh()).append ("<br>");
                                    continue;
                                }

                                int n = reportManageService.uploadFile(file, reportUploadVO, PoiUtil.ADDRESS);
                                if (0 != n) {
                                    ReportManageEO reportE = new ReportManageEO();
                                    reportE.setBgbh(reportUploadVO.getBgbh());
                                    reportE.setCpmc(reportUploadVO.getCpmc());
                                    reportE.setCpxh(reportUploadVO.getCpxh());
                                    reportE.setCpsb(reportUploadVO.getCpsb());
                                    reportE.setXmmc(reportUploadVO.getXmmc());

                                    reportE.setReportType(list.get(5));
                                    reportEs.add(reportE);
                                    reportSuccess.append (reportUploadVO.getBgbh()).append ("<br>");
                                }
                                break;
                            }
                        }
                        // 报告文件未找到
                        if(!findReport){
                            reportFailure.append ("上传文件中未找到模板中标记的报告，报告号：").append (reportUploadVO.getBgbh()).append ("<br>");
                        }
                    }
                }
            }
            } catch(Exception e){
                Result.error("批量上传失败，请联系管理员.");
                logger.error(e.getMessage(), e);
                return Result.error("r0072", "文件存储失败，请重试");
            }
        if (reportFailure.length ()>0){
            reportSuccess.append ("失败上传报告情况：<br>").append (reportFailure);
        }
        return Result.success(reportSuccess.toString ());
    }

    @PostMapping("deleteByBgbh")
    @ApiOperation(value="删除")
    public ResponseMessage deleteByBgbh(HttpServletRequest request, @RequestParam("bgbhs") String[] bgbhs) {
        Map<String,Object> usermap=(Map<String, Object>) request.getSession().getAttribute("usermap");

        if (null == usermap || usermap.isEmpty()) {
            return Result.error("请先登录系统后再进行申请.");
        }

        String qyid=usermap.get("USERNAME").toString();
        String qymc=usermap.get("NAME").toString();

        if (null == usermap || usermap.isEmpty()) {
            return Result.error("请先登录系统后再进行申请.");
        }

        int n=reportManageService.deleteByBgbh(bgbhs, qymc);
        if (0 != n) {
            return Result.success();
        } else {
            return Result.error("报告不允许删除.");
        }
    }

    @PostMapping("updateConfirmByBgbh")
    @ApiOperation("确认状态")
    public ResponseMessage updateConfirmByBgbh(HttpServletRequest request, @RequestParam("bgbhs") String[] bgbhs) {
        Map<String,Object> usermap=(Map<String, Object>) request.getSession().getAttribute("usermap");

        if (null == usermap || usermap.isEmpty()) {
            return Result.error("请先登录系统后再进行申请.");
        }
        String qyid=usermap.get("USERNAME").toString();
        String qymc=usermap.get("NAME").toString();


        int n=reportManageService.updateConfirmByBgbh(bgbhs, qymc);
        if (0 != n) {
            return Result.success();
        } else {
            return Result.error("所选择报告的状态是已确认，不用重复确认！");
        }
    }

    @RequestMapping(value = "/downloadReports", method = RequestMethod.GET)
    @ApiOperation("批量下载")
    public void downloadReports(HttpServletResponse response, String [] reportNames, String [] reportPaths) {
        //存放--服务器上zip文件的目录
        File directoryFile = new File(PoiUtil.DIRECTORY);
        if (!directoryFile.isDirectory() && !directoryFile.exists()) {
            directoryFile.mkdirs();
        }
        //设置最终输出zip文件的目录+文件名
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy年MM月dd日HH时mm分ss秒");
        String zipFileName = formatter.format(new Date()) + ".zip";
        String strZipPath = PoiUtil.DIRECTORY + File.separator + zipFileName;

        ZipOutputStream zipStream = null;
        FileInputStream zipSource = null;
        BufferedInputStream bufferStream = null;
        URL url = null;
        HttpURLConnection httpUrl = null;

        File zipFile = new File(strZipPath);
        try {
            //构造最终压缩包的输出流
            zipStream = new ZipOutputStream(new FileOutputStream(zipFile));
            for (int i = 0; i < reportPaths.length; i++) {
                //解码获取真实路径与文件名
                String realFileName = java.net.URLDecoder.decode(reportNames[i], "UTF-8");
                String realFilePath = java.net.URLDecoder.decode(reportPaths[i], "UTF-8");

                // 带后缀的文件名
                realFileName = realFilePath.substring(realFilePath.lastIndexOf("/") + 1, realFilePath.length());

                // 获取实际路径（新报告）
                if(realFilePath.contains("reportPath")){
                    realFilePath = PoiUtil.ADDRESS + realFileName;

                    logger.info("************realFilePath****************"+realFilePath);
                    File file = new File(realFilePath);

                    if (file.exists()) {
                        zipSource = new FileInputStream(file);//将需要压缩的文件格式化为输入流
                        /**
                         * 压缩条目不是具体独立的文件，而是压缩包文件列表中的列表项，称为条目，就像索引一样这里的name就是文件名,
                         * 文件名和之前的重复就会导致文件被覆盖
                         */
                        ZipEntry zipEntry = new ZipEntry(realFileName);//在压缩目录中文件的名字
                        zipStream.putNextEntry(zipEntry);//定位该压缩条目位置，开始写入文件到压缩包中
                        bufferStream = new BufferedInputStream(zipSource, 1024 * 10);
                        int read = 0;
                        byte[] buf = new byte[1024 * 10];
                        while ((read = bufferStream.read(buf, 0, 1024 * 10)) != -1) {
                            zipStream.write(buf, 0, read);
                        }
                    }
                    // 老报告读取URL方式
                } else {
                    if (PoiUtil.FileIsExist(realFilePath)) {

                        ZipEntry zipEntry = new ZipEntry(realFileName);//在压缩目录中文件的名字
                        zipStream.putNextEntry(zipEntry);//定位该压缩条目位置，开始写入文件到压缩包中

                        int BUFFER_SIZE = 1024;
                        byte[] buf = new byte[BUFFER_SIZE];
                        int size = 0;
                        url = new URL(realFilePath);
                        httpUrl = (HttpURLConnection) url.openConnection();
                        httpUrl.connect();

                        bufferStream = new BufferedInputStream(httpUrl.getInputStream());
                        while ((size = bufferStream.read(buf)) != -1) {
                            zipStream.write(buf, 0, size);
                        }
                        zipStream.flush();
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            //关闭流
            try {
                if (null != bufferStream){
                    bufferStream.close();
                }
                if (null != zipStream) {
                    zipStream.flush();
                    zipStream.close();
                }
                if (null != zipSource) {
                    zipSource.close();
                }
                if (null != httpUrl){
                    httpUrl.disconnect();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        //判断系统压缩文件是否存在：true-把该压缩文件通过流输出给客户端后删除该压缩文件  false-未处理
        if (zipFile.exists()) {
            ZipUtil.downReport(response, zipFileName, strZipPath);
            zipFile.delete();
        }
    }

    @GetMapping("/searchJceBg")
    @ApiOperation("查看检测报告")
    public ResponseMessage<String> searchJceBg(String bgbh, String reportPath) throws Exception {
        return Result.success(reportManageService.searchJceBg(bgbh, reportPath));
    }

    @GetMapping("/getBgPath")
    @ApiOperation("获取检测报告路径")
    public ResponseMessage<String> getBgPath(String bgbh) throws Exception {
        return Result.success(reportManageService.getBgPath(bgbh));
    }
    
    /**
     * 下载文件
     *
     * @param response
     * @param reportName
     * @param reportPath
     * @return 成功下载文件，失败返回0
     * @throws IOException
     */
    @RequestMapping(value = "/downloadReport", method = RequestMethod.POST)
    @ApiOperation("预览报告")
    public void downloadFile(HttpServletRequest request, HttpServletResponse response, String reportName, String reportPath) throws IOException {
        OutputStream stream = response.getOutputStream();
        FileInputStream reportInputStream= null;
        FileOutputStream reportOutputStream = null;
        BufferedInputStream bufferInputStream = null;
        BufferedOutputStream bufferOutPutStream = null;

        URL url = null;
        HttpURLConnection httpUrl = null;
        String desFilePath = null;
        String desFileName = null;

        Map<String,Object> usermap=(Map<String, Object>) request.getSession().getAttribute("usermap");
        String qymc=usermap.get("NAME").toString();

        if (reportManageService.findBgCount(qymc, reportName) == 0) {
            throw new AdcDaBaseException ("您下载的报告非本企业报告，无法下载！");
        }

        try {
            //解码获取真实路径与文件名
            String realFileName = java.net.URLDecoder.decode(reportName, "UTF-8");
            String realFilePath = java.net.URLDecoder.decode(reportPath, "UTF-8");

            // 带后缀的文件名
            realFileName = realFilePath.substring(realFilePath.lastIndexOf("/") + 1, realFilePath.length());
            desFileName = realFileName;
            desFilePath = PoiUtil.TEMPURL + realFileName;

            // 获取实际路径（新报告）
            if(realFilePath.contains("reportPath")){
                realFilePath = PoiUtil.ADDRESS + realFileName;

                logger.info("************realFilePath****************"+realFilePath);
                File file = new File(realFilePath);

                if (file.exists()) {
                    bufferInputStream = new BufferedInputStream(new FileInputStream(realFilePath));
                    bufferOutPutStream = new BufferedOutputStream(new FileOutputStream(desFilePath));

                    //写入数据
                    int BUFFER_SIZE = 1024;
                    byte[] buf = new byte[BUFFER_SIZE];
                    int size = 0;
                    while ((size = bufferInputStream.read(buf)) != -1) {
                        bufferOutPutStream.write(buf, 0, size);
                    }
                    bufferOutPutStream.flush();
                }
                // 老报告读取URL方式
            } else {
                if (PoiUtil.FileIsExist(realFilePath)) {

                    int BUFFER_SIZE = 1024;
                    byte[] buf = new byte[BUFFER_SIZE];
                    int size = 0;
                    url = new URL(realFilePath);
                    httpUrl = (HttpURLConnection) url.openConnection();
                    httpUrl.setRequestMethod("GET");
                    httpUrl.connect();

                    bufferInputStream = new BufferedInputStream(httpUrl.getInputStream());
                    bufferOutPutStream = new BufferedOutputStream(new FileOutputStream(desFilePath));

                    while ((size = bufferInputStream.read(buf)) != -1) {
                        bufferOutPutStream.write(buf, 0, size);
                    }
                    bufferOutPutStream.flush();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            //关闭流
            try {
                if (null != bufferInputStream){
                    bufferInputStream.close();
                }
                if (null != bufferOutPutStream){
                    bufferOutPutStream.flush();
                    bufferOutPutStream.close();
                }
                if (null != reportOutputStream) {
                    reportOutputStream.flush();
                    reportOutputStream.close();
                }
                if (null != reportInputStream) {
                    reportInputStream.close();
                }
                if (null != httpUrl){
                    httpUrl.disconnect();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        if (new File(desFilePath).exists()) {
            ZipUtil.downReport(response, desFileName, desFilePath);
        }
    }

}
