package com.adc.da.bs.reportManage.service;

import com.adc.da.bs.reportManage.dao.ReportManageDao;
import com.adc.da.bs.reportManage.entity.ReportManageVO;
import com.adc.da.bs.reportManage.entity.ReportUploadVO;
import com.adc.da.bs.reportManage.utils.PoiUtil;
import com.adc.da.util.exception.AdcDaBaseException;
import com.adc.da.util.http.PageParam;
import com.adc.da.util.utils.FileUtil;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ProjectName: ADCCVT
 * @Package: com.adc.cvt.FileManage.service
 * @ClassName: FileManageService
 * @Author: wanghui
 * @Description: 文件管理service
 * @Date: 2020/9/25 10:20
 * @Version: 1.0
 */

@Service
public class ReportManageService {

    private static Logger log = Logger.getLogger(ReportManageService.class);

    @Autowired
    private ReportManageDao reportManageDao;

    /**
     * @param page
     * @return
     * @Method queryListByPage
     * @Author wanghui
     * @Version 1.0
     * @Description 分页查询
     * @Return java.lang.Void
     * @Exception
     * @Date 2020/9/25 10:21
     */
    public Map<String, Object> queryListByPage(PageParam page, ReportManageVO reportManageVO) {
        if (StringUtils.isEmpty(reportManageVO.getQyid())) {
            throw new AdcDaBaseException("缺少当前企业信息，无法查询");
        }
        Map<String, Object> result = new HashMap<>();
        //先根据条件计算总数
        Long count = reportManageDao.findTotalCount(page, reportManageVO);
        List<Map<String, Object>> list = reportManageDao.queryListByPage(page, reportManageVO);

        // 获取报告的实际后缀名
        // 查询原始文件名称
        for (Map<String, Object> map : list) {
            String jgdh = null == map.get("JGDH") ? "" : map.get("JGDH").toString();
            String bgbh = null == map.get("BGBH") ? "" : map.get("BGBH").toString();
            String reportPath = null == map.get("REPORTPATH") ? "" : map.get("REPORTPATH").toString();

            // 针对新报告
            if (StringUtils.isEmpty(jgdh)) {
                if (FileUtil.exists(PoiUtil.ADDRESS + bgbh + ".pdf")) {
                    reportPath = reportPath + ".pdf";
                } else if (FileUtil.exists(PoiUtil.ADDRESS + bgbh + ".PDF")) {
                    reportPath = reportPath + ".PDF";
                } else if (FileUtil.exists(PoiUtil.ADDRESS + bgbh + ".doc")) {
                    reportPath = reportPath + ".doc";
                } else {
                    reportPath = reportPath + ".DOC";
                }
                // 针对老报告
            } else {

                try {
                    if(PoiUtil.FileIsExist(reportPath + ".pdf")){
                        reportPath = reportPath + ".pdf";
                    }
                } catch (Exception e) {
                    try {
                        if (PoiUtil.FileIsExist (reportPath + ".PDF")) {
                            reportPath = reportPath + ".PDF";
                        }
                    } catch (Exception ex) {
                        try {
                            if (PoiUtil.FileIsExist (reportPath + ".doc")) {
                                reportPath = reportPath + ".doc";
                            }
                        } catch (Exception ex1) {
                            ex1.printStackTrace ();
                        }
                    }
                }
            }
            // 返回带后缀名的全路径报告
            map.put("REPORTPATH", reportPath);
        }
        /*Map<String, Object> map = reportManageDao.queryDetail(id);
        String file_name = null == map.get("FILE_NAME") ? "" : map.get("FILE_NAME").toString();
        File file = new File(address + file_name);
        //文件存在 - 删除原始文件
        if (file.exists()) {
            System.out.println("原始文件存在！删除原始文件！" + address + file_name);
            file.delete();
        } else {
            System.out.println("原始文件不存在！文件路径：" + address + file_name);
        }*/

        result.put("total", count);
        result.put("rows", list);
        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    public int uploadFile(MultipartFile file, ReportUploadVO reportUploadVO, String ADDRESS) {
        int i = 0;
        try {
            if (null != file) {
                //判断目标路径是否存在
                File fileUpload = new File(ADDRESS);
                if (!fileUpload.exists()) {
                    //不存在 创建目录
                    fileUpload.mkdirs();
                }
                System.out.println("文件名称：" + file.getOriginalFilename());
                //创建上传文件对象（文件存储的路径，文件名称）
                fileUpload = new File(ADDRESS, file.getOriginalFilename());

                //将文件写入到服务器中
                file.transferTo(fileUpload);
                i = reportManageDao.saveReportDetail(reportUploadVO);
            }
            return i;
        } catch (IOException e) {
            log.error(e.getMessage(), e);
            return 0;
        } finally {
        }
    }

    /**
     * 删除服务器上原始文件
     *
     * @param id
     * @param address
     */
    public void dealOriginalFile(String id, String address) {
        //查询原始文件名称
        /*Map<String, Object> map = reportManageDao.queryDetail(id);
        String file_name = null == map.get("FILE_NAME") ? "" : map.get("FILE_NAME").toString();
        File file = new File(address + file_name);
        //文件存在 - 删除原始文件
        if (file.exists()) {
            System.out.println("原始文件存在！删除原始文件！" + address + file_name);
            file.delete();
        } else {
            System.out.println("原始文件不存在！文件路径：" + address + file_name);
        }*/
    }

    /**
     * @param bgbhs
     * @Method updateConfirmByBgbh
     * @Author deng
     * @Version 1.0
     * @Description 修改确认状态
     * @Return void
     * @Exception
     * @Date 2020/10/21 13:37
     */
    @Transactional
    public int updateConfirmByBgbh(String[] bgbhs, String qymc) {
        int i = 0;
        int j = 0;
        for (String bgbh : bgbhs) {
            if (bgbh == null || "".equals(bgbh) || "null".equals(bgbh)) {
                throw new AdcDaBaseException("报告编号不存在！无法确认！！！");
            }
            j = reportManageDao.updateConfirmByBgbh(bgbh, qymc);
            i = i + j;
        }
        return i;
    }

    /**
     * @param bgbhs
     * @Method deleteByBgbh
     * @Author deng
     * @Version 1.0
     * @Description 删除
     * @Return void
     * @Exception
     * @Date 2020/10/27 13:49
     */
    @Transactional
    public int deleteByBgbh(String[] bgbhs, String qyid) {
        int i = 0;
        int j = 0;
        for (String bgbh : bgbhs) {
            if (bgbh == null || "".equals(bgbh) || "null".equals(bgbh)) {
                throw new AdcDaBaseException("报告编号不存在！无法删除！！！");
            }
            j = reportManageDao.deleteByBgbh(bgbh, qyid);
            i = i + j;
        }
        return i;
    }

    public List<String> checkBgkStatus(String bgbh) {

        List<String> result = reportManageDao.checkBgkStatus(bgbh);
        return result;
    }

    public String searchJceBg(String bgbh, String reportPath) {
        String url = reportManageDao.searchJceBg(bgbh, reportPath);
        return url;
    }

    public int findBgCount(String qymc, String bgbh) {
        Long count = reportManageDao.findBgCount(qymc, bgbh);
        return count.intValue();
    }

	/**   
	 * @Title: getBgPath   
	 * @Description: TODO(这里用一句话描述这个方法的作用)    
	 * @param @param bgbh
	 * @param @return    设定文件   
	 * @return String    返回类型   
	 * @author ADC_ZH 
	 * @date 上午11:46:29 
	*/   
	public String getBgPath(String bgbh) {
		String path ="";
		
        List<Map<String, Object>> list = reportManageDao.getBgPath(bgbh);
        if(list.size()>0){
        	Map<String, Object> map = list.get(0);
        	path =map.get("BGPATH")!=null?map.get("BGPATH").toString():"";
            String jgdh = null == map.get("JGDH") ? "" : map.get("JGDH").toString();
         // 针对新报告
            if (StringUtils.isEmpty(jgdh)) {
                if (FileUtil.exists(PoiUtil.ADDRESS + bgbh + ".pdf")) {
                	path = path + ".pdf";
                } else if (FileUtil.exists(PoiUtil.ADDRESS + bgbh + ".PDF")) {
                	path = path + ".PDF";
                } else if (FileUtil.exists(PoiUtil.ADDRESS + bgbh + ".doc")) {
                	path = path + ".doc";
                } else {
                	path = path + ".DOC";
                }
                // 针对老报告
            } else {
                try {
                    if(PoiUtil.FileIsExist(path + ".pdf")){
                    	path = path + ".pdf";
                    }
                } catch (Exception e) {
                    try {
                        if (PoiUtil.FileIsExist (path + ".PDF")) {
                        	path = path + ".PDF";
                        }
                    } catch (Exception ex) {
                        try {
                            if (PoiUtil.FileIsExist (path + ".doc")) {
                            	path = path + ".doc";
                            }
                        } catch (Exception ex1) {
                            ex1.printStackTrace ();
                        }
                    }
                }
            }
        }
		return path;
	}
}
