package com.adc.da.bs.resourceSharing.rest;

import com.adc.da.bs.resourceSharing.entity.QaAnswerEntity;
import com.adc.da.bs.resourceSharing.entity.QaExpertsLibFileEntity;
import com.adc.da.bs.resourceSharing.entity.QaIssueEntity;
import com.adc.da.bs.resourceSharing.service.QandAService;
import com.adc.da.communal.Utils.DateUtils;
import com.adc.da.util.exception.AdcDaBaseException;
import com.adc.da.util.http.PageParam;
import com.adc.da.util.utils.FileUtil;
import com.adc.da.util.utils.RedisUtils;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.omg.CORBA.ObjectHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;

import java.io.IOException;
import java.io.InputStream;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**资源共享  问答模块 包含 提问  问题集 经典问题
 * @author ZhengZhiwei
 * @description
 * @date create at 15:38 2018/3/29
 * @modified by
 */
@Controller
@RequestMapping(value = "/${restPath}/qa")
@Api(description = "资源共享问答模块")
public class QandAController {
    private final Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private QandAService qandAService;
    @Value("${file.path}")
	private String basePath;//默认路径
    
    @Autowired
    private RedisUtils redisUtils;
    /**
     * @describtion TODO 问题集列表
     * @param
     * @author ZhengZhiwei
     * @date create at 15:57 2018/3/29
     * @modified by
     */
    @PostMapping("/list")
    @ResponseBody
    @ApiOperation(value = "经验共享--最新问题2021")
    public Map<String, Object> findQaList(HttpServletRequest request, PageParam page, String param, 
    		@ApiParam(value = "UN:未回答，myQ:我的提问，myA我的回答") @RequestParam(required=false) String isA, 
    		@ApiParam(value = "ZR：准入，SB:申报，XCP:新产品，BG:变更，KZ:扩展, RP:报告") @RequestParam(required=false) String qtype){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        if(isA != null && usermap == null){
            errMap.put("err", "当前用户未登录");
            return errMap;
        }
        Map<String,Object> qaList = qandAService.findQaList(page,usermap, isA, param,qtype);
        return qaList;
    }
    /**
     * @describtion TODO 问题集列表
     * @param
     * @author Zhong
     * @date create at 15:57 2018/5/31
     * @modified by
     */
    @PostMapping("/list_index")
    @ResponseBody
    @ApiOperation(value = "问题集列表")
    public Map<String, Object> list_index(HttpServletRequest request, PageParam page,String type, String param){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        if(type != null && usermap == null){
            errMap.put("err", "当前用户未登录");
            return errMap;
        }
        page.setLimit(5);
        page.setOffset(0);
        Map<String,Object> qaList = qandAService.findQaList(page,usermap, type, param,"");
        return qaList;
    }
  /**
   * @author ZhengZhiwei
   * @describtion TODO 新建问题
   * @param
   * @date create at 8:12 2018/4/2
   * @modified by
   */
    @SuppressWarnings("unchecked")
    @PostMapping("/addQaIssue")
    @ResponseBody
    @ApiOperation(value = "新建问题")
    public Map<String, Object> addQaIssue(HttpServletRequest request,@RequestBody QaIssueEntity qaIssue){
    	
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        if(usermap == null){
            errMap.put("err", "当前用户未登录");
            return errMap;
        }
        int VIRTUAL_POINT=Integer.parseInt(usermap.get("VIRTUAL_POINT")==null?"0":usermap.get("VIRTUAL_POINT").toString());
        //积分减去默认5分加特殊奖励分数
        VIRTUAL_POINT= VIRTUAL_POINT-5-qaIssue.getRewards();
        Map<String,Object> result =  new HashMap<>();
        /*if(VIRTUAL_POINT>=0){
        	usermap.put("VIRTUAL_POINT", VIRTUAL_POINT);
        	request.getSession().setAttribute("usermap", usermap);
        	result =  qandAService.addQaIssue(usermap, qaIssue);
        }else{
        	result.put("err", "积分不足");
        }*/
    	request.getSession().setAttribute("usermap", usermap);
    	result =  qandAService.addQaIssue(usermap, qaIssue);
        return result;
    }
    
    /**
     * @describtion TODO top3
     * @param
     * @author Zhanghong
     * @date create at 2018/6/4
     * @modified by
     */
    @PostMapping("/top3")
    @ResponseBody
    @ApiOperation(value = "top3")
    public List<Map<String, Object>> top3(HttpServletRequest request){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        List<Map<String, Object>> result = qandAService.getTopNSore(3);
        return result;
    }
    
    /**
     * @describtion TODO 单个问题详情
     * @param
     * @author Zhanghong
     * @date create at 2018/6/7
     * @modified by
     */
    @PostMapping("/loadQDetailByid")
    @ResponseBody
    @ApiOperation(value = "单个问题详情-2021")
    public Map<String, Object> loadSingleQaissueDataByQid(HttpServletRequest request, String qid){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        Map<String, Object> res = qandAService.findIssueByQid(usermap, qid);
        //判断是否在前台可以选择最佳答案
        if(usermap==null||usermap.isEmpty()){
        	res.put("select", false);
        }else{
        	try {
				boolean self=Integer.parseInt(usermap.get("ID").toString())==Integer.parseInt(((Map<String, Object>)res.get("res")).get("USERID").toString());
				res.put("select", self);//如果为自己的问题可以选择最佳答案
			} catch (Exception e) {
				res.put("select", false);
			}
        }
        return res;
    }
    /**
     * @describtion TODO 选为最佳答案
     * @param
     * @author Zhanghong
     * @modified by
     */
    @SuppressWarnings({ "unchecked", "unused", "rawtypes" })
	@PostMapping("/bestanswer")
    @ResponseBody
    @ApiOperation(value = "选为最佳答案-2021")
    public Map<String, Object> bestanswer(HttpServletRequest request, String qid, String aid){
    	
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        if(usermap==null){
            throw new AdcDaBaseException("未登录！");
		};
        Map<String, Object> res = qandAService.bestanswer(usermap, qid,aid);
        if(res.get("QA_POINT")!=null)usermap.put("QA_POINT", res.get("QA_POINT").toString());//给session中积分重新赋值
        return res;
    }
    /**
     * @describtion TODO 取消最佳答案
     * @param
     * @author Zhanghong
     * @modified by
     */
    @SuppressWarnings({ "unchecked", "unused", "rawtypes" })
	@PostMapping("/cancelbestanswer")
    @ResponseBody
    @ApiOperation(value = "取消最佳答案-2021")
    public Map<String, Object> cancelbestanswer(HttpServletRequest request, String qid, String aid){
    	
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        Map<String, Object> res = qandAService.cancelbestanswer(usermap, qid,aid);
        return res;
    }
    /**
     * @author ZhengZhiwei
     * @describtion TODO 回答
     * @param 
     * @return 
     * @date create at 15:42 2018/4/13
     * @modified by
     */
    @PostMapping("/addQaAnswer")
    @ResponseBody
    @ApiOperation(value = "添加问题回答")
    public Map<String, Object> addQaAnswer(HttpServletRequest request, @RequestBody QaAnswerEntity qaAnswer){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        if(usermap == null){
            errMap.put("err", "当前用户未登录");
            return errMap;
        }
        Map<String, Object> res = qandAService.addQaAnswer(usermap, qaAnswer);
        return res;
    }
    /**
     * 
     * @Title: addQaReply   
     * @Description: TODO(这里用一句话描述这个方法的作用)    
     * @param @param request
     * @param @param qaAnswer
     * @param @return    设定文件    
     * @author ADC_ZH 
     * @date 上午10:18:49
     */
    @PostMapping("/addQaReply")
    @ResponseBody
    @ApiOperation(value = "添加回复(面向答案对话)-2021")
    public Map<String, Object> addQaReply(HttpServletRequest request, @RequestBody QaAnswerEntity qaAnswer){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        if(usermap == null){
            errMap.put("err", "当前用户未登录");
            return errMap;
        }
        Map<String, Object> res = qandAService.addQaAnswer(usermap, qaAnswer);
        return res;
    }
    /**
     * 
     * @Title: addQaReply   
     * @Description: TODO(这里用一句话描述这个方法的作用)    
     * @param @param request
     * @param @param qaAnswer
     * @param @return    设定文件   
     *    返回类型   
     * @author ADC_ZH 
     * @date 上午10:18:49
     */
    @PostMapping("/getQaReplyList")
    @ResponseBody
    @ApiOperation(value = "获取回复列表(面向答案对话)-2021")
    public Map<String, Object> getQaReplyList(HttpServletRequest request, String  aid){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        if(usermap == null){
            errMap.put("err", "当前用户未登录");
            return errMap;
        }
        Map<String, Object> res = qandAService.getQaReplyList(usermap, aid);
        return res;
    }
    
    /**
     * 
     * @Title: addQaReply   
     * @Description: TODO(这里用一句话描述这个方法的作用)    
     * @param @param request
     * @param @param qaAnswer
     * @param @return    设定文件   
     *    返回类型   
     * @author ADC_ZH 
     * @date 上午10:18:49
     */
    @PostMapping("/deleteQaAnswer")
    @ResponseBody
    @ApiOperation(value = "删除答案&回复-2021")
    public Map<String, Object> deleteQaAnswer(HttpServletRequest request, String aid){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        if(usermap == null){
            errMap.put("err", "当前用户未登录");
            return errMap;
        }
        Map<String, Object> res = qandAService.deleteQaAnswer(usermap, aid);
        return res;
    }
    
    @PostMapping("/deleteQaIssue")
    @ResponseBody
    @ApiOperation(value = "删除问题-2021")
    public Map<String, Object> deleteQaIssue(HttpServletRequest request, String aid){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        if(usermap == null){
            errMap.put("err", "当前用户未登录");
            return errMap;
        }
        Map<String, Object> res = qandAService.deleteQaIssue(usermap, aid);
        return res;
    }
    /**
     * @author ZhengZhiwei
     * @describtion TODO 点赞数量修改
     * @param 
     * @return 
     * @date create at 15:55 2018/4/13
     * @modified by
     */
    @PostMapping("/pushPraiseOrNot")
    @ResponseBody
    @ApiOperation(value = "点赞数量修改-2021")
    public Map<String, Object> pushPraiseOrNot(HttpServletRequest request, @ApiParam(value = "Y:赞; N:踩")  @RequestParam(required=false) String type, String aid, String qid){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        if(usermap == null){
            logger.error(DateUtils.getDate("yyyy-MM-dd HH:mm:ss", new Date())+ "用户未登录");
            throw new AdcDaBaseException("未登录用户，请先登陆！");
        }
        if(StringUtils.isNotEmpty(type)&& "Y".equals(type)){//先将以前的赞或踩取消掉
        	Map<String, Object> resa = qandAService.cancelPraiseOrNot(usermap, "N", aid, qid);
        }else if(StringUtils.isNotEmpty(type)&& "N".equals(type)){        	
        	Map<String, Object> resa = qandAService.cancelPraiseOrNot(usermap, "Y", aid, qid);
        }

        Map<String, Object> res = qandAService.pushPraiseOrNot(usermap, type, aid, qid);
        return res;
    }
    
    /**
     * 
     * @Title: cancelPraiseOrNot   
     * @Description: TODO(取消点赞)    
     * @param @param request
     * @param @param type
     * @param @param aid
     * @param @param qid
     * @param @return    设定文件   
     *    返回类型   
     * @author ADC_ZH 
     * @date 上午11:09:54
     */
    @PostMapping("/cancelPraiseOrNot")
    @ResponseBody
    @ApiOperation(value = "取消点赞/踩-2021")
    public Map<String, Object> cancelPraiseOrNot(HttpServletRequest request, @ApiParam(value = "Y:赞; N:踩")  @RequestParam(required=false) String type, String aid, String qid){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        if(usermap == null){
            logger.error(DateUtils.getDate("yyyy-MM-dd HH:mm:ss", new Date())+ "用户未登录");
            throw new AdcDaBaseException("未登录用户，请先登陆！");
        }
        Map<String, Object> res = qandAService.cancelPraiseOrNot(usermap, type, aid, qid);
        return res;
    }

    /**
     * @author ZhengZhiwei
     * @describtion TODO 获取经典问题列表
     * @param
     * @return
     * @date create at 10:11 2018/4/16
     * @modified by
     */
    @PostMapping("/findClassicQList")
    @ResponseBody
    @ApiOperation(value = "备案经验列表-2021")
    public Map<String, Object> findClassicQList(HttpServletRequest request, PageParam page,String type, String param){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");

        Map<String,Object> res = qandAService.findClassicQList(page,usermap, type, param);
        return res;
    }

    /**
     * @author ZhengZhiwei
     * @describtion TODO 上传经典问题
     * @param
     * @return
     * @date create at 10:11 2018/4/16
     * @modified by
     */
    @PostMapping("/uploadClassicQ")
    @ResponseBody
    @ApiOperation(value = "上传经典问题")
    public Map<String, Object> uploadClassicQ(HttpServletRequest request,String enterType, @RequestBody QaExpertsLibFileEntity qaExpertsLibFile){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        if(usermap == null){
            errMap.put("err", "当前用户未登录");
            return errMap;
        }
        Map<String,Object> res = qandAService.uploadClassicQ(usermap,enterType, qaExpertsLibFile);
        return res;
    }
    /**
     * @author ZhengZhiwei
     * @describtion TODO 获取经典问题的详细信息
     * @param
     * @return
     * @date create at 10:11 2018/4/16
     * @modified by
     */
    @PostMapping("/findClassicQDetail")
    @ResponseBody
    @ApiOperation(value = "备案经验详情-2021")
    public Map<String, Object> findClassicQDetail(HttpServletRequest request, String param){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");

        Map<String,Object> res = qandAService.findClassicQDetail(usermap, param);
        return res;
    }
    /**
     * @describtion TODO 经典问题列表
     * @param
     * @author Zhanghong
     * @modified by
     */
    @PostMapping("/getTopScoreUser")
    @ResponseBody
    @ApiOperation(value = "获取经典问题列表")
    public Map<String, Object> getTopScoreUser(HttpServletRequest request){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        if(usermap == null){
            errMap.put("err", "当前用户未登录");
            return errMap;
        }
        Map<String,Object> res = qandAService.getTopScoreUser();
        return res;
    }
    /**
     * @describtion TODO 获取未读消息数量
     * @param
     * @author Zhanghong
     * @modified by
     */
    @PostMapping("/countQamessage")
    @ResponseBody
    @ApiOperation(value = "获取未读消息数量")
    public Map<String, Object> countQamessage(HttpServletRequest request){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        if(usermap == null){
            errMap.put("err", "当前用户未登录");
            return errMap;
        }
        Map<String,Object> res = qandAService.countQamessage(usermap.get("ID").toString());
        return res;
    }
    /**
     * @describtion TODO 未读消息列表
     * @param
     * @author Zhanghong
     * @modified by
     */
    @PostMapping("/qamessageList")
    @ResponseBody
    @ApiOperation(value = "未读消息列表")
    public Map<String, Object> qamessageList(HttpServletRequest request, PageParam page,String type, String param){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        if(type != null && usermap == null){
            errMap.put("err", "当前用户未登录");
            return errMap;
        }
        Map<String,Object> qaList = qandAService.qamessageList(page,usermap.get("ID").toString());
        return qaList;
    }
    /**
     * @describtion TODO 标记已读消息
     * @param
     * @author Zhanghong
     * @modified by
     */
    @PostMapping("/upQamessage")
    @ResponseBody
    @ApiOperation(value = "标记已读消息")
    public Map<String, Object> upQamessage(HttpServletRequest request,String id){
        Map<String,Object> errMap = new HashMap<>();
        Map<String, Object> usermap = (Map)request.getSession().getAttribute("usermap");
        if(usermap == null){
            errMap.put("err", "当前用户未登录");
            return errMap;
        }
        Map<String,Object> qaList = qandAService.upQamessage(id,usermap.get("ID").toString());
        return qaList;
    }
    
    @PostMapping("/orderQyname")
    @ResponseBody
    @ApiOperation(value = "企业贡献排序列表-2021")
    public Map<String, Object> orderQyname(HttpServletRequest request){
        Map<String,Object> res = new HashMap<>();
        if(redisUtils.exists("orderQyname")){
            res=(Map<String, Object>) redisUtils.get("orderQyname");
		}else{
			res = qandAService.orderQyname();
			long min=60*24*7;  
			redisUtils.set("orderQyname" , res, min, TimeUnit.MINUTES);
		}
        return res;
    }
    
    @PostMapping("/qaListByHot")
    @ResponseBody
    @ApiOperation(value = "热门问题列表-2021")
    public Map<String, Object> qaListByHot(HttpServletRequest request ,PageParam page){
        Map<String,Object> res = new HashMap<>();
        res = qandAService.qaListByHot(page);
			 
        return res;
    }
    
/*    
    @PostMapping("/orderByHot")
    @ResponseBody
    @ApiOperation(value = "更新热门排序-2021")
//  @Scheduled(cron = "0 10 17 12 * ?")
    public void orderByHot(HttpServletRequest request){ 
    	System.out.println("===========================定时任务--更新热门排序！===========================");
    	qandAService.orderByHot(); 
    }*/

}
