package com.adc.da.bs.resourceSharing.service;

import com.adc.da.bs.resourceSharing.dao.QandADao;
import com.adc.da.bs.resourceSharing.entity.QaAnswerEntity;
import com.adc.da.bs.resourceSharing.entity.QaExpertsLibFileEntity;
import com.adc.da.bs.resourceSharing.entity.QaIssueEntity;
import com.adc.da.communal.Utils.DateUtils;
import com.adc.da.util.exception.AdcDaBaseException;
import com.adc.da.util.http.PageParam;
import com.adc.da.util.utils.RedisUtils;

import org.apache.commons.lang3.StringUtils;
import org.omg.PortableInterceptor.INACTIVE;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import javax.servlet.http.HttpServletRequest;

/**资源共享  问答模块 包含 提问  问题集 经典问题
 * @author ZhengZhiwei
 * @description
 * @date create at 15:45 2018/3/29
 * @modified by
 */
@Service
public class QandAService {
    @Autowired
    private QandADao qandADao;
    @Autowired
    private RedisUtils redisUtils;
    /**
     * @describtion TODO 问题集列表
     * @param
     * @author ZhengZhiwei
     * @date create at 15:57 2018/3/29
     * @modified by
     */
    public Map<String, Object> findQaList(PageParam page,Map<String, Object> usermap,String isA, String param, String qtype) {
        //当前用户id
        Integer entId = 0;
        if (usermap != null && !usermap.isEmpty()) {
            entId = Integer.parseInt(usermap.get("ID").toString());
        }
        Map<String, Object> result = new HashMap<>();
        Long count=qandADao.findTotalCount(entId,isA,param, qtype);
        List<Map<String, Object>> list=qandADao.findListByPage(page,entId,isA,param, qtype);
        for (Map<String, Object> map :list) {

            //匿名
            if(map.containsKey("ANONYMITY") && map.get("ANONYMITY") !=null &&StringUtils.equals(map.get("ANONYMITY").toString(),"Y")) {
                //如果用户未登陆
                if (usermap == null || usermap.isEmpty()) {
                    map.put("NAME", "匿名用户");
                } else {
                    boolean isAdmin = StringUtils.equals("admin", usermap.get("QA_TYPE")==null?"":usermap.get("QA_TYPE").toString());
                    // 不是当前用户的问题 不是管理员
                    if (!StringUtils.equals(entId.toString(), map.get("ENTID").toString()) && !isAdmin) {
                        map.put("NAME", "匿名用户");
                    }
                }
            }
            //添加时间（2分钟前，2天前...）
            String dateStr =  map.get("CREATDATE").toString();
            Date date = DateUtils.stringToDate("yyyy-MM-dd HH:mm:ss",dateStr);
            String timeGap = DateUtils.dateToString(date);
            map.put("TIME", timeGap);
        }
 
        result.put("total", count);
        result.put("rows" , list);
        if(redisUtils.exists("countByType")){
			result.put("countByType" , redisUtils.get("countByType"));

		}else{ 
			List<Map<String, Object>> countByType=qandADao.countByType();
			Map<String, Object> mapcountByType=new HashMap<>();
			for(Map<String, Object> m :countByType){
				mapcountByType.put(m.get("QTYPE")==null?"-":m.get("QTYPE").toString(), m.get("COUNT")==null?"0":m.get("COUNT").toString());
			}
			result.put("countByType" , mapcountByType);
			long min=60*24*1;  
			redisUtils.set("countByType" , mapcountByType, min, TimeUnit.MINUTES);
		}
        return result;
    }

    /**
     * @author ZhengZhiwei
     * @describtion TODO 新建问题
     * @param
     * @date create at 8:12 2018/4/2
     * @modified by
     */
    @Transactional
    public Map<String,Object> addQaIssue(Map<String, Object> usermap, QaIssueEntity qaIssueEntity) {
        int userId = Integer.parseInt(usermap.get("ID").toString());
        int entId = userId;
        qaIssueEntity.setUserid(userId);
        qaIssueEntity.setRetrievalId(entId);
        qaIssueEntity.setReplyCount(0);
        qaIssueEntity.setViewCount(0);
        //新增问题数据
        Map<String, Object> result = qandADao.addQaIssue(qaIssueEntity);
//        int VIRTUAL_POINT=Integer.parseInt(usermap.get("VIRTUAL_POINT").toString());
        //根据用户id修改积分
//        qandADao.changePointByUid(-5,-5-qaIssueEntity.getRewards(),userId);
        List<Map<String, Object>> countByType=qandADao.countByType();
        Map<String, Object> mapcountByType=new HashMap<>();
        for(Map<String, Object> m :countByType){
        	mapcountByType.put(m.get("QTYPE")==null?"-":m.get("QTYPE").toString(), m.get("COUNT")==null?"0":m.get("COUNT").toString());
        }
//		redisUtils.set("countByType" , mapcountByType);
		long min=60*24*1;  
		redisUtils.set("countByType" , mapcountByType, min, TimeUnit.MINUTES);

        return result;
    }


	public List<Map<String, Object>> getTopNSore(int i) {
		List<Map<String, Object>> result = qandADao.getTopNSore(i);
        return result;
	}


    /**
     * @author ZhengZhiwei
     * @describtion TODO 获取问题详情
     * @param 
     * @return 
     * @date create at 15:50 2018/4/12
     * @modified by
     */
    @Transactional
    public Map<String, Object> findIssueByQid(Map<String, Object> usermap, String qid){
        Map<String, Object> res = new HashMap<>();
        if(StringUtils.isEmpty(qid)){
            res.put("err","访问失败，请稍后重试");
            return res;
        }

        if(!qandADao.changeViewCountByQid(qid)){
            res.put("err","访问失败，请稍后重试");
            return res;
        }
        List<Map<String, Object>> qaList = qandADao.getIssueById(qid);
        List<Map<String, Object>> answerList = qandADao.getAnswersByQid(qid);
        Map<String, Object> qa = new HashMap<>();
        if(qaList.size() == 1){
            qa = qaList.get(0);
            qa.put("answerList", answerList);
            //匿名
            if(qa.containsKey("ANONYMITY") && qa.get("ANONYMITY") !=null  && StringUtils.equals(qa.get("ANONYMITY").toString(),"Y")) {
                //如果用户未登陆
                if (usermap == null || usermap.isEmpty()) {
                    qa.put("NAME", "匿名用户");
                } else {
                    Integer entId = Integer.parseInt(usermap.get("ID").toString());
                    boolean isAdmin = StringUtils.equals("admin", usermap.get("QA_TYPE")==null?"":usermap.get("QA_TYPE").toString());
                    // 不是当前用户的问题 不是管理员
                    if (!StringUtils.equals(entId.toString(), qa.get("ENTID").toString()) && !isAdmin) {
                        qa.put("NAME", "匿名用户");
                    }
                }
            }

            res.put("res", qa);
        }else{
            res.put("err","访问失败，请稍后重试");
            return res;
        }

        return res;
    }

    /**
     * @author ZhengZhiwei
     * @describtion TODO 添加回答
     * @param 
     * @return 
     * @date create at 12:36 2018/4/13
     * @modified by
     */
    @Transactional
    public Map<String,Object> addQaAnswer(Map<String, Object> usermap, QaAnswerEntity qaAnswerEntity) {
        Map<String, Object> res = new HashMap<>();
        //将回复记录添加到QA_MESSAGE 表中
        String content = "您的问题得到了回复！";
        String type = "回复";
        String flag = "N";
        //获取提问人的userId和提问人所属企业的企业id
        List<Map<String, Object>> mUserList = qandADao.getMuserIdByQid(qaAnswerEntity.getQid());
        String m_userid = null;
        String m_entid = null;
        Integer msg_userid = null;
        Integer msg_entid = null;
        if(!mUserList.isEmpty()){
            m_userid = mUserList.get(0).get("USERID")==null?"":mUserList.get(0).get("USERID").toString();
            m_entid = mUserList.get(0).get("ENTID")==null?"":mUserList.get(0).get("ENTID").toString();
        }
        if(StringUtils.isNotEmpty(m_userid) && StringUtils.isNotEmpty(m_entid)){
            msg_userid = Integer.parseInt(m_userid);
            msg_entid = Integer.parseInt(m_entid);
        }

        //答题人的信息
        Integer userid = Integer.parseInt(usermap.get("ID")==null?"":usermap.get("ID").toString());
        String enterpriseName = usermap.get("NAME").toString();
        qaAnswerEntity.setUserid(userid);
        qaAnswerEntity.setRetrievalId(userid);
        qaAnswerEntity.setEnterpriseName(enterpriseName);
        qaAnswerEntity.setGoodCount(0);
        qaAnswerEntity.setBadCount(0);
        //添加msg
        boolean isSaveMsg = qandADao.saveAnswerToMessage(content,type,qaAnswerEntity.getUserid(),qaAnswerEntity.getQid(),flag,qaAnswerEntity.getRetrievalId(),msg_userid,msg_entid);
        //保存回答
        boolean bool = qandADao.addQaAnswer(qaAnswerEntity);
        if(bool){
            res.put("res",true);
            return res;
        }else{
            res.put("err","访问失败，请稍后重试");
            return res;
        }

    }

    /**
     * @author ZhengZhiwei
     * @describtion TODO 点赞数量修改
     * @param
     * @return
     * @date create at 15:54 2018/4/13
     * @modified by
     */
    @Transactional
    public Map<String,Object> pushPraiseOrNot(Map<String, Object> usermap, String type, String aid, String qid) {
        Map<String, Object> res = new HashMap<>();
        Integer id = Integer.parseInt(qid);
        if(usermap.get("ID")== null){
            res.put("err","访问失败，请稍后重试");
            return res;
        }
        String uid = usermap.get("ID").toString();
        //将点赞记录添加到QA_MESSAGE 表中
        String content = "您的问题得到了点赞！";
        String type_dianzan = "点赞";
        String flag = "N";
        Integer userid = Integer.parseInt(uid);
        Integer entid =  Integer.parseInt(uid);
        List<Map<String,Object>> mUserList = this.qandADao.getMuserIdByAid(Integer.parseInt(aid));
        String m_userid = null;
        String m_entid = null;
        Integer msg_userid = null;
        Integer msg_entid = null;
        if(!mUserList.isEmpty()){
            m_userid = mUserList.get(0).get("USERID")==null?"":mUserList.get(0).get("USERID").toString();
            m_entid = mUserList.get(0).get("ENTID")==null?"":mUserList.get(0).get("ENTID").toString();
        }
        if(StringUtils.isNotEmpty(m_userid) && StringUtils.isNotEmpty(m_entid)){
            msg_userid = Integer.parseInt(m_userid);
            msg_entid = Integer.parseInt(m_entid);
        }

        boolean isSaveA2M = qandADao.saveAnswerToMessage(content, type_dianzan, userid, id, flag, entid, msg_userid, msg_entid);
        boolean isChangeC = qandADao.changePraiseCount(type, aid);
        if(isChangeC){
            //记录用户
            qandADao.addPraiseHistory(aid, uid,type);
        }else{
            res.put("err","访问失败，请稍后重试");
            return res;
        }
        res.put("res","success");
        return res;

    }

    /**
     * @author ZhengZhiwei
     * @describtion TODO 经典问题列表
     * @param 
     * @return 
     * @date create at 10:20 2018/4/16
     * @modified by
     */
    public Map<String,Object> findClassicQList(PageParam page, Map<String, Object> usermap, String type, String param) {
        Integer entId = Integer.parseInt(usermap == null ? "0":usermap.get("ID").toString());
        Map<String, Object> result = new HashMap<>();
        Long count=qandADao.findClassicQCount(page,entId,type,param);
        List<Map<String, Object>> list=qandADao.findClassicQList(page,entId,type,param);
        for (Map<String, Object> map :list) {

            //添加时间（2分钟前，2天前...）
            String dateStr =  map.get("UPLOADDATE").toString();
            String timeGap = "";
            if(dateStr != null) {
                Date date = DateUtils.stringToDate("yyyy-MM-dd HH:mm:ss", dateStr);
                timeGap = DateUtils.dateToString(date);
            }
            map.put("TIME", timeGap);
        }

        result.put("total", count);
        result.put("rows" , list);

		if(redisUtils.exists("countByType")){
 			result.put("countByType" , redisUtils.get("countByType"));

		}else{ 
			List<Map<String, Object>> countByType=qandADao.countByType();
			Map<String, Object> mapcountByType=new HashMap<>();
			for(Map<String, Object> m :countByType){
				mapcountByType.put(m.get("QTYPE")==null?"-":m.get("QTYPE").toString(), m.get("COUNT")==null?"0":m.get("COUNT").toString());
			}
			result.put("countByType" , mapcountByType);

			long min=60*24*1;  
			redisUtils.set("countByType" , mapcountByType, min, TimeUnit.MINUTES);
//			redisUtils.set("countByType" , mapcountByType);
		}
		
        return result;
    }

    /**
     * @author ZhengZhiwei
     * @describtion TODO 上传经典问题
     * @param
     * @return
     * @date create at 10:11 2018/4/16
     * @modified by
     */
    @Transactional
    public Map<String,Object> uploadClassicQ(Map<String, Object> usermap,String enterType, QaExpertsLibFileEntity qaExpertsLibFile) {
        Integer userid = Integer.parseInt(usermap.get("ID").toString());//也是entid

        qaExpertsLibFile.setUserid(userid);
        qaExpertsLibFile.setRetrievalId(userid);
        qaExpertsLibFile.setAccepted("");
        Map<String,Object> res = this.qandADao.saveNewExpertsLibFileData(enterType, qaExpertsLibFile);
        //添加到QA_MESSAGE 表中
        String content = qaExpertsLibFile.getUserid()+"企业,上传了资料！";
        String type_dianzan = "业务资料上传";
        String ywzl_flag = qaExpertsLibFile.getRetrievalId()+"";
        Integer entid =  qaExpertsLibFile.getRetrievalId();
        Integer qid =null;
        if(res.containsKey("qid")){
            qid = Integer.parseInt(res.get("qid").toString());
        }

        this.qandADao.saveYwzlToMessage(content, type_dianzan, userid, qid, entid, ywzl_flag);
        return res;
    }

    /**
     * @author ZhengZhiwei
     * @describtion TODO 获取经典问题的详细信息
     * @param 
     * @return 
     * @date create at 7:56 2018/4/17
     * @modified by
     */
    @Transactional
    public Map<String,Object> findClassicQDetail(Map<String, Object> usermap, String param) {
        qandADao.changeViewCountByCQid(param);
        Map<String,Object>  res = qandADao.findClassicQDetail(param);
        return res;
    }

    public Map<String,Object> findClassicQFileInfo(String id,String type) {
        if(StringUtils.equals("qFile",type)){
            return qandADao.findQFileInfo(id);
        }else if(StringUtils.equals("cqFile",type)){
            return qandADao.findClassicQFileInfo(id);
        }else{
            return qandADao.findQAFileInfo(id);
        }

    }

    /**
     * @author ZhengZhiwei
     * @describtion TODO 获取用户排行
     * @param 
     * @return 
     * @date create at 8:50 2018/4/17
     * @modified by
     */
    public Map<String, Object> getTopScoreUser(){
        Map<String, Object> res = new HashMap<>();
        List<Map<String, Object>> resList= qandADao.getTopScoreUser("3");
        res.put("res", resList);
        return res;
    }
    /**
     * @author Zhanghong
     * @describtion TODO 选取最佳答案并修改积分
     * @param 
     * @return 
     * @date create at 8:50 2018/5/15
     * @modified by
     */
    @Transactional
	public Map<String, Object> bestanswer_old(Map<String, Object> usermap,
			String qid, String aid) {
    	
    	Map<String, Object> res = new HashMap<>();
		try {
			Map<String, Object> question=qandADao.getIssueById(qid).get(0);
			//获取奖励分数
			int point=Integer.parseInt(question.get("REWARDS").toString());
			//根据答案ID获取答案
			Map<String, Object> answer=qandADao.getMuserIdByAid(Integer.parseInt(aid)).get(0);
			//获取回答者ID
			int uid=Integer.parseInt(answer.get("USERID").toString());
			//更改回答者积分
			qandADao.changePointByUid(5+point,5+point,uid);
			//获取作者实际积分
			int QA_POINT=Integer.parseInt(usermap.get("QA_POINT").toString())-point;
			//更改作者实际积分
			qandADao.changePointByUid(-point,0,Integer.parseInt(usermap.get("USERID").toString()));
			qandADao.bestanswer(qid,aid);//更改问题状态
			res.put("re", 1);
			res.put("QA_POINT", QA_POINT);
		} catch (Exception e) {
			res.put("re", 0);
		}
		return res;
	}
    /**
     * @author Zhong
     * @describtion TODO 未读消息数量
     * @param 
     * @return 
     * @date create at 8:50 2018/6/7
     * @modified by
     */
	public Map<String, Object> countQamessage(String uid) {
		Map<String, Object> res = new HashMap<>();
		Integer count =qandADao.findUnreadMsg(uid);
		res.put("count", count);
		return res;
	}
	/**
     * @author Zhong
     * @describtion TODO 未读消息列表
     * @param 
     * @return 
     * @date create at 8:50 2018/6/8
     * @modified by
     */
	public Map<String, Object> qamessageList(PageParam page, String uid) {
		Map<String, Object> result = new HashMap<>();
		List<Map<String, Object>> list=qandADao.qamessageList(page, uid);
		Integer count=qandADao.findUnreadMsg(uid);
		result.put("total", count);
		result.put("rows" , list);
		return result;
	}
    @Transactional
	public Map<String, Object> upQamessage(String id, String uid) {
		Map<String, Object> result = new HashMap<>();
		boolean count=qandADao.qamessageUp(id,uid);
		result.put("count", count);
		return result;
	}

	/**   
	 * @Title: cancelPraiseOrNot   
	 * @Description: TODO(这里用一句话描述这个方法的作用)    
	 * @param @param usermap
	 * @param @param type
	 * @param @param aid
	 * @param @param qid
	 * @param @return    设定文件   
	 *    返回类型   
	 * @author ADC_ZH 
	 * @date 上午11:10:20 
	*/   
    @Transactional
	public Map<String, Object> cancelPraiseOrNot(Map<String, Object> usermap,
			String type, String aid, String qid) {
        Map<String, Object> res = new HashMap<>();
        Integer id = Integer.parseInt(qid);
        if(usermap.get("ID")== null){
            res.put("err","访问失败，请稍后重试");
            return res;
        }
        String uid = usermap.get("ID").toString();
        /*//将点赞记录添加到QA_MESSAGE 表中
        String content = "您的问题得到了点赞！";
        String type_dianzan = "点赞";
        String flag = "N";
        Integer userid = Integer.parseInt(uid);
        Integer entid =  Integer.parseInt(uid);
        List<Map<String,Object>> mUserList = this.qandADao.getMuserIdByAid(Integer.parseInt(aid));
        String m_userid = null;
        String m_entid = null;
        Integer msg_userid = null;
        Integer msg_entid = null;
        if(!mUserList.isEmpty()){
            m_userid = mUserList.get(0).get("USERID")==null?"":mUserList.get(0).get("USERID").toString();
            m_entid = mUserList.get(0).get("ENTID")==null?"":mUserList.get(0).get("ENTID").toString();
        }
        if(StringUtils.isNotEmpty(m_userid) && StringUtils.isNotEmpty(m_entid)){
            msg_userid = Integer.parseInt(m_userid);
            msg_entid = Integer.parseInt(m_entid);
        }

        boolean isSaveA2M = qandADao.saveAnswerToMessage(content, type_dianzan, userid, id, flag, entid, msg_userid, msg_entid);*/
        boolean isChangeC = qandADao.cancelPraiseCount(type, aid,uid);
        if(isChangeC){
            //记录用户
            qandADao.cancelPraiseHistory(aid, uid,type);
        }else{
            res.put("err","访问失败，请稍后重试");
            return res;
        }
        res.put("res","success");
        return res;

    
	}

	/**   
	 * @Title: cancelbestanswer   
	 * @Description: TODO(这里用一句话描述这个方法的作用)    
	 * @param @param usermap
	 * @param @param qid
	 * @param @param aid
	 * @param @return    设定文件   
	 *    返回类型   
	 * @author ADC_ZH 
	 * @date 下午4:40:28 
	*/   
    @Transactional
	public Map<String, Object> bestanswer(Map<String, Object> usermap,
			String qid, String aid) {
		if(usermap.get("ID")== null){
			 throw new AdcDaBaseException("取消最佳答案失败！");
        }
        String uid = usermap.get("ID").toString();
        Map<String, Object> res = new HashMap<>();
		if(qandADao.cancelbestanswer(uid,qid,aid)){
			qandADao.bestanswer(uid,qid,aid);//更改问题状态		
		}else{
			 throw new AdcDaBaseException("取消最佳答案失败！");
		};

	    return res;
	}

	/**   
	 * @Title: bestanswer   
	 * @Description: TODO(这里用一句话描述这个方法的作用)    
	 * @param @param usermap
	 * @param @param qid
	 * @param @param aid
	 * @param @return    设定文件   
	 *    返回类型   
	 * @author ADC_ZH 
	 * @date 下午4:49:44 
	*/   
    @Transactional
	public Map<String, Object> cancelbestanswer(Map<String, Object> usermap,
			String qid, String aid) {
		String userId =usermap.get("ID").toString();
		Map<String, Object> res = new HashMap<>();
		if(qandADao.cancelbestanswer(userId,qid,aid)){
			res.put("res","success");			
		}else{
			 throw new AdcDaBaseException("取消最佳答案失败！");
		};
	    return res;
	}

	/**   
	 * @Title: getQaReplyList   
	 * @Description: TODO(这里用一句话描述这个方法的作用)    
	 * @param @param usermap
	 * @param @param aid
	 * @param @return    设定文件   
	 *    返回类型   
	 * @author ADC_ZH 
	 * @date 下午2:11:34 
	*/   
	public Map<String, Object> getQaReplyList(Map<String, Object> usermap,
			String aid) {
		if(usermap.get("ID")== null){
			 throw new AdcDaBaseException("未登录！");
       }
       String uid = usermap.get("ID").toString();
       Map<String, Object> res = new HashMap<>();
		List<Map<String, Object>> list=qandADao.getQaReplyList(uid, aid); 
		res.put("rows" , list);
	    return res;
	}

	/**   
	 * @Title: deleteQaAnswer   
	 * @Description: 删除回答、回复
	 * @param @param usermap
	 * @param @param aid
	 * @param @return    设定文件   
	 *    返回类型   
	 * @author ADC_ZH 
	 * @date 下午2:23:15 
	*/
    @Transactional   
	public Map<String, Object> deleteQaAnswer(Map<String, Object> usermap,
			String aid) {
    	if(usermap.get("ID")== null){
			 throw new AdcDaBaseException("未登录！");
      }
      String uid = usermap.get("ID").toString();
		Map<String, Object> res = new HashMap<>();
		if(qandADao.deleteQaAnswer(uid,aid)){			
			res.put("res","success");	
			List<Map<String, Object>> countByType=qandADao.countByType();
	        Map<String, Object> mapcountByType=new HashMap<>();
	        for(Map<String, Object> m :countByType){
	        	mapcountByType.put(m.get("QTYPE")==null?"-":m.get("QTYPE").toString(), m.get("COUNT")==null?"0":m.get("COUNT").toString());
	        } 
			long min=60*24*2;  
			redisUtils.set("countByType" , mapcountByType, min, TimeUnit.MINUTES);
		}else{
			 throw new AdcDaBaseException("删除失败！");
		}
		return res;
	}
    /**   
	 * @Title: deleteQaIssue   
	 * @Description: 删除问题
	 * @param @param usermap
	 * @param @param aid
	 * @param @return    设定文件   
	 *    返回类型   
	 * @author ADC_ZH 
	 * @date 下午2:23:15 
	*/
    @Transactional   
	public Map<String, Object> deleteQaIssue(Map<String, Object> usermap,
			String aid) {
    	if(usermap.get("ID")== null){
			 throw new AdcDaBaseException("未登录！");
      }
      String uid = usermap.get("ID").toString();
		Map<String, Object> res = new HashMap<>();
		if(qandADao.deleteQaIssue(uid,aid)){	
			List<Map<String, Object>> countByType=qandADao.countByType();
			Map<String, Object> mapcountByType=new HashMap<>();
	        for(Map<String, Object> m :countByType){
	        	mapcountByType.put(m.get("QTYPE")==null?"-":m.get("QTYPE").toString(), m.get("COUNT")==null?"0":m.get("COUNT").toString());
	        } 
			long min=60*24*2;  
			redisUtils.set("countByType" , mapcountByType, min, TimeUnit.MINUTES);
			res.put("res","success");			
		}else{
			 throw new AdcDaBaseException("删除失败！");
		}
		return res;
	}
	/**   
	 * @Title: orderQyname   
	 * @Description: TODO(这里用一句话描述这个方法的作用)    
	 * @param @return    设定文件   
	 *    返回类型   
	 * @author ADC_ZH 
	 * @date 上午10:55:28 
	*/   
	public Map<String, Object> orderQyname() {
		Map<String, Object> result = new HashMap<>();
		List<Map<String, Object>> orderQyname=qandADao.orderQyname(); 
		result.put("orderQyname" , orderQyname);
	    return result;
	}

	/**   
	 * @Title: orderByHot   
	 * @Description: TODO(这里用一句话描述这个方法的作用)    
	 * @param     设定文件   
	 * @return void    返回类型   
	 * @author ADC_ZH 
	 * @date 上午11:43:08 
	*/   
	public void orderByHot() {
		// TODO Auto-generated method stub
		
	}

	/**   
	 * @Title: qaListByHot   
	 * @Description: TODO(这里用一句话描述这个方法的作用)    
	 * @param @return    设定文件   
	 *    返回类型   
	 * @author ADC_ZH 
	 * @date 上午11:58:26 
	*/   
	public Map<String, Object> qaListByHot(PageParam page) {
		Map<String, Object> result = new HashMap<>();
		List<Map<String, Object>> qaListByHot=qandADao.qaListByHot(page); 
		result.put("qaListByHot" , qaListByHot);
        Long count=qandADao.findTotalCount(null,"","","");
		result.put("total" , count);

		if(redisUtils.exists("countByType")){
			result.put("countByType" , redisUtils.get("countByType"));
		}else{ 
			List<Map<String, Object>> countByType=qandADao.countByType();
			Map<String, Object> mapcountByType=new HashMap<>();
			for(Map<String, Object> m :countByType){
				mapcountByType.put(m.get("QTYPE")==null?"-":m.get("QTYPE").toString(), m.get("COUNT")==null?"0":m.get("COUNT").toString());
			}
			result.put("countByType" , mapcountByType);
//			redisUtils.set("countByType" , mapcountByType);
			 
			long min=60*24*1;  
			redisUtils.set("countByType" , mapcountByType, min, TimeUnit.MINUTES);
		}
		
	    return result;
	}
}
